/**.
 *  Fetching a component of a page.
 *  The details of the component such as URL are provided with ComponentDetails object. 
 *  Socket is created to the server/proxy (depending on configuration) and the request is sent. 
 *  The received data is analyzed. If the content type is text/html, then image tags are searched
 *  The image URL is retrieved and it is fetched in similar way
 *  5th Nov - Adding header analysis part
 *  25th Nov - Adding various timing parameters 
 */


import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import com.quiotix.html.parser.*;
import com.quiotix.html.parser.HtmlDocument.Attribute;
import com.quiotix.html.parser.HtmlDocument.AttributeList;


class Component extends Thread implements HttpHeadGlobal 
{
	Thread t;
	ComponentResult		CRLocal;
	ComponentDetails	CDLocal;	
	NumThread		obSynch;	// object for synchronization

/*
 * The constructor receives the details of component in object CD.
 * The results of the component are to be stored in object CR.
 * ob1 is the object for synchronizing the exit of the component
 * 
 */

	Component(ComponentDetails CD, ComponentResult CR, NumThread ob1)
	{
		t = new Thread(this);
		t.setName("Component");
		CDLocal = CD;
		CRLocal = CR; 
		obSynch = ob1; 	// synchronization object
	//	System.out.println(" Component	: " + CDLocal.cmpURL);
		t.start();
	}

	public void run()
	{
		String		Request = new String("");	// HTTP  Request to be sent to server
		String 		cmpData = new String("");	// component content 
		String  	cmpHeader = new String("");	// HTTP header associated with component
		Date 		D1;				// variable for time measurement
		long 		T1, T2, T3, T4,T5; 		// variable for time measurement
		int 		conDel = 0;			// connect delay
		int 		sndreq = 0;			// send request delay
		int 		fb =0;				// first byte latency
		InputStream	rd;				// stream for reading from socket
		OutputStream 	wr;				// stream for writing to socket
		int 		n; 
		final int	MAX_SIZE = 4096;		
		byte 		buff[]= new byte[MAX_SIZE]; 	// buffer to read the data from socket  
		int 		headSize = 0; 			// header size
		int 		dataSize = 0;			// data size  which is the size of the component

	
		//get time stamp
		D1 = new Date();	
		T1 = D1.getTime();

		Request = new String(ConstructReq());	// construct the http request for fetching the page

		// open the conenction to server or proxy (depending on configuration)
		OwnSocket S = new OwnSocket(CDLocal.cmpURL , CDLocal.cmpPort);

		if (S.isErr()) // check if there is any error in creating socket
		{

			CRLocal.setFailure(S.getErr());
			CRLocal.cmpURL		=  CDLocal.cmpURL;	// URL of the component 
			obSynch.exitThread();
			return;	
		}
	
		// get time stamp for measuring connect delay
		D1 = new Date();
		T2 = D1.getTime();

		conDel = (int) (T2 - T1);		// connect delay noted

		// === Connection Established. Read the response.
		try
		{
			rd = S.s.getInputStream();
			wr = S.s.getOutputStream();
			wr.write(Request.getBytes());	// write the http request to socket

			D1 = new Date();
			T3 = D1.getTime();
			sndreq = (int) (T3 - T2); 	// "send request"  delay

			// flag for measuring first byte latency. 
			// once the latency is measured (i.e. after first read()), flag is set to 1
			int flagfirst = 0;	
			while ((n=rd.read(buff,0,MAX_SIZE)) >= 0)    // start reading the data
			{
				if (flagfirst == 0)
				{
					flagfirst = 1;
					D1 = new Date();
					T4  = D1.getTime();
					fb = (int) (T4 - T3);	// note the first byte latency
				}
				cmpData +=  new String(buff,0,n);
			} //end while
			S.s.close();
		}
		catch (Exception e)
		{
		// 	System.out.println("Exception in getting page "+ e);
			/**. Get error message correctly. 
			 *	The exception string is
			 *		+ The name of the actual class of this object 
			 *		+ ": " (a colon and a space) 
			 *		+ The result of the getMessage() method for this object 
			 *	We just want the name of the class. Remaining components are removed.
			 *	Also, at the beginning of each class, the hierarchy of the classes is presented. 
			 *	The hierarchy is eliminated and only final class name is retrieved. 
			 *	Eg. If the message is "java.net.UnknownHostException: vsnlprox.iitk.ac.in", 
			 *		only UnknownHostException is retrieved.
			 */
			System.out.println("Component	: Exception " + e);
			
			String err      = new String();
			err             = e.toString();
			String str2     = err.substring(0, err.indexOf(':'));	// remove the colon and further message

			String str3     = str2.substring(str2.lastIndexOf('.')+1); // remove hierarchy except last component

			CRLocal.setFailure(str3);	// set final error message
			CRLocal.cmpURL		=  CDLocal.cmpURL;	// URL of the component 
			obSynch.exitThread();
			try
			{
				S.s.close();
			}
			catch (IOException ioe)
			{

			}
			return;	
		}
		
		D1 = new Date();		// get time stamp for complete download delay
		T5 = D1.getTime();

		// === Complete data received
		// === Analyze the header for more information
		
		// Extract the header from recived data
		cmpHeader 	= getHeader(cmpData);	

		// note the header length
		headSize	+= cmpHeader.length();		

		// get complete data size
		// note: data includes header. hence the actual component size is dataSize minus headSize
		dataSize	+= cmpData.length();
	
		// System.out.println("Component :\n"+cmpHeader);
		
		// *** Analyzing header....
		String	statLine = new String("");	// Status Line - contains response code	
		String	respCode = new String("");	// HTTP Response code
		int	conLen;				// Length of the response body (in bytes)
		String	conType = new String("");	// Response Content-type
		int 	anal1=0, anal2=0; 		// index variable for analysis

		// Retrieve HTTP response code
		anal1 = cmpHeader.indexOf(Delim);
		statLine = cmpHeader.substring(0,anal1);	// get the status line
		anal1 = statLine.indexOf(" "); 	
		anal2 = statLine.indexOf(" ", anal1+1);
		respCode = statLine.substring(anal1+1, anal2);	// get the HTTP response code


		if (respCode.equals("200"))	// everything is fine 
		{	
			// check the content type. 
			// if it is "text/html" find out the image tag in it and fetch it.
		
			conType = getHeadParam(cmpHeader, "Content-Type");
			// System.out.println("Component: "+ CDLocal.cmpURL + " Content " + conType);

			/* content is html code - search for image tag
			 * Algorithm : 
			 * 1. check content type is "text/html"
			 * 2. if yes, get the file name from the URL of the component. Add the current date at the begining of the file
			 * 3. create the file with above file name
			 * 4. write the data of the component to the file
			 * 5. find the img tag if any
			 *
			 */

			if (conType.equalsIgnoreCase("text/html"))
			{
				int 	ind=0;
				String 	cFname = new String("");
				File	f;

				//find the file name from the URL
				ind = CDLocal.cmpURL.lastIndexOf('/');
				cFname = new String(CDLocal.cmpURL.substring(ind+1));

				/*  add the current date at the begining of the file name
				 *  here date means the a number representing that date
				 *  this number is equivalent of the number of milli seconds from 1st Jan 1970 to the current date/time.
				 */
				Date D = new Date();
				long L = D.getTime();
				cFname = L + cFname;
			
				// create new  file with name Fname
				f = new File(cFname);
				try
				{
					f.createNewFile();
				}
				catch (Exception e)
				{
					System.out.println(" Error in creating file  " + e );
				}
	
				// write the contents to file
				try
				{	
					FileWriter out = new FileWriter(cFname, true); // parameter is file name and true is for append

					// write without http header 
					out.write(cmpData,  cmpHeader.length() , cmpData.length() - cmpHeader.length()); 
					cmpData	 = "";		// free the memory
					cmpHeader= "";
					out.flush();
					out.close();
				}
				catch (Exception e)
				{
					System.out.println(" exception in writing page to file " + e);	
				}

				// find the images in the component
				// read the file 
				// get img tags
				// add to a list
				// ASSUMPTION: At most one image URL will be found in the component
			
				ArrayList	cImgName = new ArrayList();

				try 
				{
					
					FileInputStream in = new FileInputStream(cFname);
					HtmlDocument document;

					document = new HtmlParser((InputStream)in).HtmlDocument();
					document.accept(new GetImgName(System.out, cImgName));
					in.close();
				}
				catch (Exception e)
				{
					System.out.println(" Exception in opening file Exception " + e);
				}
			} // end if text/html
			cmpData = "";
		}// end response code == 200

		
		// response code starts with 3 - redirection 
		if (respCode.startsWith("3") ==  true )	
		{
			String	newURL = new String("");

			// get the new URL from "Location" field of the header
			CDLocal.cmpURL = getHeadParam(cmpHeader, "Location") ;	// Fetch the new URL
		}
		
		// Response code starts with 4 - client error 
		if (respCode.startsWith("4"))
		{
			// System.out.println("Component: Client Error " + respCode +"  for "+ CDLocal.cmpURL ); 

			CRLocal.setFailure(respCode);			// set Response code received as error message 
			CRLocal.cmpURL		=  CDLocal.cmpURL;	// URL of the component 
			obSynch.exitThread();
			return;	
		}

		// response code starts with 5 - server error 
		if (respCode.startsWith("5"))
		{
			//System.out.println("Server Error"); 

			CRLocal.setFailure(respCode);			// set Response code received as error message 
			CRLocal.cmpURL		=  CDLocal.cmpURL;	// URL of the component 
			obSynch.exitThread();
			return;	
		}

		CRLocal.cmpURL		=  CDLocal.cmpURL;	 

		// get the right content-type
		if (conType.regionMatches(true,0,"text",0,4))
			CRLocal.contentType	=  new String("TXT");	// content is text
		if (conType.regionMatches(true,0,"image",0,5))
			CRLocal.contentType	=  new String("IMG");	// content is image 
		if (conType.regionMatches(true,0,"video",0,5))
			CRLocal.contentType	=  new String("VID");	// content is video
		if (conType.regionMatches(true,0,"audio",0,5))
			CRLocal.contentType	=  new String("AUD");	// content is audio

		CRLocal.connectDelay 	=  conDel ;
		CRLocal.sendReqDelay	=  sndreq ;
		CRLocal.FBLatency	=  fb;
		CRLocal.cmpDL		=  (int) (T5 - T1);
		CRLocal.headerSize 	=  headSize; 
		CRLocal.cSize		=  dataSize - headSize;
		// CRLocal.headerSize 	=  cmpHeader.length();
		// CRLocal.cSize	=  cmpData.length() - cmpHeader.length();

		obSynch.exitThread();
		// System.out.println("** component " + CDLocal.cmpURL + " done...value of num "+ obSynch.curVal());
		return;
	}

	/** If proxy is being used then every request which is in the form
	 *	http://www.somesite.com/path/to/page.htm 
	 *	is sent in following format
	 * 	http://www.somesite.com:<portnumber>/path/to/page.htm 
	 *
	 */
	private String ProxyReq(String tmpURL, int port)
	{
		int	index = 0;	
		String	finStr = new String("");
		index = tmpURL.indexOf('/',8);  // the URL starts with "http://", hence it is ingnored
		
		finStr = tmpURL.substring(0, index);
		finStr += ":" + port;
		finStr += tmpURL.substring(index);
		return finStr;
	}

	
	private String ConstructReq()
	{
		String Tmp = new String("");		// String holding temporary request	
		String Httpv = new String("");		// HTTP Version for the page
		
		boolean	proxyStatus = Global.proxyStatus(CDLocal.cmpURL);	// check the proxy status for component URL

		Tmp = GET;	// use HTTP Get Method
		Httpv = HTTP10;	// HTTP version to be 1.0

		if (proxyStatus == true)  // proxy is being used then provide the port number with URL
		{	Tmp += ProxyReq(CDLocal.cmpURL, CDLocal.cmpPort) ; }
		else
		{	Tmp += CDLocal.cmpURL; }
		
		Tmp += Httpv + Delim; 	// add HTTP version

		if (proxyStatus == true)  // proxy is being used - provide authentication
		{	
			// use proxy authorization only if the authorization string is present. 
			if (Global.authUser.length() > 0) 
				Tmp += ProxyAuth + Global.authUser; 
		}
		Tmp +=  EndReq;

		return Tmp;
	}

	/** 
	 *  getheader extracts out the http header from the received data
	 *  if the sourece string has complete header, header is returned 
	 *  else an empty string will be returned.  
	 */

	private String getHeader(String Src)
	{
		String 	HSrc = Src;		// local variable for parameter 1
		String 	HHead = new String("");	 	// local variable for parameter 2
		int	i;			// index variable 
		int	itemp =0, itemp2=0;	// temporary index variables
		String 	Hcomp = new String("");	// header component		

		/* The header fields are seperated by "\r\n" 
		 * The header finishes with a blank line 
		 * So a blank line is searched as the end of header 
		 */
 
		while (true)
		{
			itemp = HSrc.indexOf(Delim,itemp2);
			if (itemp < 0) return HHead;
			Hcomp = HSrc.substring(itemp2,itemp);

			// empty string indicates header has been received completely
			if (Hcomp.equals("")) 
			{		
				HHead = new String(HSrc.substring(0, itemp+2));
				return HHead;
			}
			itemp2 = itemp + 2;
		}
	}

/* 	
 * getHeadParam returns the value for the particular parameter in HTTP header 
 * The input is complete Header string and the parameter whose value is to be retrieved
 */
	private String getHeadParam(String head, String paramVal)
	{
		String 	res = new String("");		// result string
		int anal1=0, anal2=0;			// index variables

		anal1 = head.indexOf(paramVal);
			
		if (anal1 < 0) // required parameter not found
		{
			return res; //	return empty string indicating failure
		} 

		anal1 =	head.indexOf(" ", anal1);
		anal2 = head.indexOf(Delim, anal1);

		if (anal2 < 0) // malformed header
		{
			return res; //	return empty string indicating failure
		} 

		res = head.substring(anal1+1, anal2);

		return res;
	}
}

