
/**.	This class does some boot-up work for delegate.
 *	Various parameters used to access Reporting Center are stored in a configuration file (rcparam.conf) 
 * 	[*** Actual location of rcparam.conf will depend on the operating system of installation]
 *	Intitially, this configuration file does not exist. 
 *	While the Delegate starts for the first time, the configuration file is created
 *	Once configuration file is created, this class does not have any function
 *	Any line starting with a '#' is considered to be comment and is ignored
 */

import java.io.File;
import java.io.FileWriter;
import java.io.*;
import java.util.*;

class ConfigDelegate
{

	final 	String BASE_DIR_CONF	= new String("/usr/local/webtest/conf/");
	final String	configFile 	= new String(BASE_DIR_CONF + "rcparam.conf");
	final String	userInfoFile 	= new String(BASE_DIR_CONF + "userinfo.conf");

	ConfigDelegate()
	{
		//==== create  configuration file

		File f = new File(configFile);
		if (f.exists() == true)	// if file exits, return
			return;

		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : ConfigDelegate.java " + e );
			}
		}

		// file created, now update the parameters

		try
		{

			BufferedWriter 	out = new BufferedWriter(new FileWriter(configFile));
			
			out.write("### Please do NOT edit this file "); // warming message
			out.newLine();	
			out.write("ResFile = trans.res ");		// Name of the file for storing transaction results
			out.newLine();	// add newline at the end
			out.write("ProfileFile = monitor.pro ");		// Name of the file for storing various profiles
			out.newLine();	
			out.write("DelID =  "+ createDelID());		// Delegate ID
			out.newLine();

			// Parameters for uploading results for transactions
			out.write("UpResURL = http://www.yourserver.com/path/to/resupload.php ");	// URL for the page
			out.newLine();
			out.write("UpResPort = 80 ");	// Port number to be used to establish connection
			out.newLine();
			out.write("UpResMethod = POST ");	// HTTP Method to be used
			out.newLine();	
			out.write("UpResVers = 1.0 ");	// HTTP Version to be used
			out.newLine();

			// Parameters for sending alive message to the Reporting Centre
			out.write("AliveURL = http://www.yourserver.com/path/to/alive.php ");	// URL for the page
			out.newLine();
			out.write("AlivePort = 80 ");	// Port number to be used to establish connection
			out.newLine();
			out.write("AliveMethod = GET ");	// HTTP Method to be used
			out.newLine();	
			out.write("AliveVers = 1.0 ");	// HTTP Version to be used
			out.newLine();

			// Parameters for downloading the job profile
			out.write("JobDLURL = http://www.yourserver.com/path/to/getjob.php ");	// URL for the page
			out.newLine();
			out.write("JobDLPort = 80 ");	// Port number to be used to establish connection
			out.newLine();
			out.write("JobDLMethod = GET ");	// HTTP Method to be used
			out.newLine();	
			out.write("JobDLVers = 1.0 ");	// HTTP Version to be used
			out.newLine();

			// Parameters for Registration of Delegates
			out.write("RegURL = http://www.yourserver.com/path/to/register.php ");	// URL for the page
			out.newLine();
			out.write("RegPort = 80 ");	// Port number to be used to establish connection
			out.newLine();
			out.write("RegMethod = POST ");	// HTTP Method to be used
			out.newLine();	
			out.write("RegVers = 1.0 ");	// HTTP Version to be used
			out.newLine();

			out.flush();
			out.close();
		}
		catch (Exception e)
		{		
			System.out.println("ConfigDelegate : Unable to write to  rcparam.conf " );
		}
	}

	/**.
	 *	This method creates an unique ID for this delegate and returns it.
	 *	DelID = Hex representation of the current time stamp 
	 */	

	private String createDelID()
	{
		Date 	d	= new Date();
		String id 	= Long.toHexString(d.getTime());

		return id;

	} // end createDelID

	/*	userInfo method creates a user info file and writes the information from the form.
	 *	This info is submitted to Reporting Centre while registration of Delegate 
	 */
	
	public void userInfo(String name, String email, String reg, String city, String inet, String os, String priv)
	{
		// === Create UserInfo file and store various information 
	
		File f = new File(userInfoFile);
		if (f.exists() == true)	// if file exits, return
			return;

		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : ConfigDelegate.java " + e );
			}
		}

		// file created, now update the parameters

		try
		{

			BufferedWriter 	out = new BufferedWriter(new FileWriter(userInfoFile));
			
			out.write("### Please do NOT edit this file "); // warming message
			out.newLine();	
			out.write("#Registered is a flag to indicate whether Delegate is Registered with Reporting Centre");
			out.newLine();

			// initial value is no indicating delegate is not registered
			// note the two spaces after "NO".
			// when Delegate is registered, it will be changed to "YES" with string of exact length
			out.write("Registered = NO  ");	
			
			out.newLine();
			out.write("Name = "+ name);		// name of the user
			out.newLine();
			out.write("Email = "+email);		// email address
			out.newLine();
			out.write("Region = "+reg);		// region of the delegate
			out.newLine();
			out.write("City = "+city);		// city of residence
			out.newLine();
			out.write("Link = "+inet);		// link to internet
			out.newLine();
			out.write("OS = "+os);			// operating system
			out.newLine();
			out.write("Priv = "+priv);		// privelege level for Delegate
			out.newLine();
			out.flush();
			out.close();
		}
		catch (Exception e)
		{		
			System.out.println("ConfigDelegate : Unable to write to rcparam.conf ");
		}
	}

	/**.
	 *	This method is to check whether Delgate is registered. 
	 *	If Delegate is already registered, there is no need for multiple registration. 
	 *	If Delegate is not registered, then the registration is done.
	 *	The registration status can be found from the userinfo.conf
	 *	
	 */

	public boolean regDel()
	{
		String	name 	= new String(""); 
		String	email	= new String(""); 
		String	region	= new String("");
		String	city	= new String("");
		String	link	= new String("");
		String	os	= new String("");
		String	priv	= new String("");

		try
		{
			BufferedReader input = new BufferedReader(new FileReader(userInfoFile)); 
			String s = new String();	// string to read file contents line-by-line

			
			// === first check whether delegate is registered
			while((s=input.readLine())!= null) 
			{
				StringTokenizer st = new StringTokenizer(s);	// creates token of the string
				String		par = st.nextToken();		// get the first token which is the parameter name
				String		eq = st.nextToken();		// second token is "=" sign
				String		val = st.nextToken();		// third token is the value of the parmeter

				if (par.startsWith("#"))			// a # indicates comment - hence ignore
					continue;
				if (par.equals("Registered"))			// Registration status
				{
					if (val.equals("YES"))			// Delegate already registered
						return true;			// return success
				}

				// reading other parameters as Delegate is not registered
				if (par.equals("Name")) 		
				{
					// in name, there can be more than one token eg."richard stallman"
					// hence all the token must be read and a space is inserted between two tokens
					name = val;	
					while (st.hasMoreTokens()) 		// read all the remaining tokens	
					{
						name += " ";
						name += st.nextToken();
					}

				}
				if (par.equals("Email"))
				{
					email = val;
				}
				if (par.equals("Region"))
				{
					region = val;
				}
				if (par.equals("City"))
				{
					// in city, there can be more than one token eg "new jersey"
					// hence all the token must be read and a space is inserted between two tokens
					city = val;
					while (st.hasMoreTokens()) 		// read all the remaining tokens	
					{
						city	+= " ";
						city	+= st.nextToken();
					}
				}
				if (par.equals("Link"))
				{
					link = val;
				}
				if (par.equals("OS"))
				{
					os = val;
				}
				if (par.equals("Priv"))
				{
					priv = val;
				}
			}// end while s=readline

			input.close();

		} // end try
		catch (Exception e)
		{
		}

		// reading all the parameters completed	
			
		// ***  Now register the Delegate

		String pgParam 	= new String("");	// page parameters
		String pgParVal	= new String("");	// values for respective page parameters
		String		delim = new String("|");	// delimeter to seperate parameters
		int 		i=0;
		PageDetails	pgDet; 
		PageResult 	pgRes;
		Page		p;
		String 		fName = new String("");	 // name of the file in which the response from Reporting Centre is stored

		/**.
		 *	Create the parameter and value strings 
		 *	The parameters to be sent while registering 
		 *	DelID	- Delegate ID
		 *	Name	- Name of the User 
		 *	Email	- Email Address of the User
		 *	DReg	- Region of Delegate
		 * 	DCity	- City of Delegate
		 * 	DLink	- Internet link used by Delegate
		 *	DOp	- Operating System on which Delegate is installed
		 * 	DProxy	- Flag to indicate use of proxy by Delegate
		 * 	DAc	- The privilege levels for the Delegate
		 * 
		 */
			 
		pgParam		= "DelID"+delim+"Name"+delim+ "Email"+delim+ "DReg"+delim;
		pgParVal	= Global.getDelID()+ delim + name + delim + email + delim + region + delim;
		pgParam		+= "DCity"+delim+ "DLink"+delim+ "DOp"+delim+ "DAc"+delim;
		pgParVal	+= city + delim + link + delim + os + delim + priv+ delim;
		pgParam		+= "DProxy"+delim;

		if (Global.proxyStatus("")) 
		{
			// proxy being used
			pgParVal	+= "YES" + delim;
		}
		else
		{
			// proxy not in use
			pgParVal	+= "NO" + delim;
		}

		pgDet = Global.getRegDetail();
		pgRes = new PageResult();
		
		pgDet.setPgParam(pgParam, pgParVal,"" );

		p = new Page(pgDet, pgRes);
		try
		{
			p.t.join();
		}
		catch (InterruptedException e)
		{	
			System.out.println("ConfigDelegate : Thread Interrupted");
		}

		fName = pgRes.pgFile;
		try
		{
			BufferedReader input = new BufferedReader(new FileReader(fName));
			String s = new String("");
	
			while((s=input.readLine())!= null) 
			{
				// check for OK string at the beginning 
				if (s.startsWith("OK"))
				{
					// everything is fine. 
					input.close();

					// Delegate is registered successfully
					// update the status of registration
					if (setRegDone())
						return true; // registration status updated successfully
					else 
						return false;
				}
			} // end while
			input.close();
		}// end try
		catch(IOException e) 
		{
			System.out.println("ConfigDelegate: IOException ");
	    	}
		return false;
	}// end regDel

/**.
 *	setRegDone method is called when Registration of delegate is completed successfully. 
 *	It sets the Registered Flag in userInfo.conf 
 *
 */
 

	synchronized private boolean setRegDone()
	{
		String	fName = new String();	
		String 	tmpFile	= new String("userInfo.tmp");

		fName = userInfoFile ;	// get result file name

		// create temporary file for storing result 
		File f = new File(tmpFile);
		if (f.exists() == false)	// if temporary file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : ConfigDelegate.java " + e );
				return false;
			}
		}
	
		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(tmpFile));
			BufferedReader in = new BufferedReader(new FileReader(fName));

			String 	s	= new String("");

			// ***	copy contents of result file in temp file except the uploaded result
			while((s=in.readLine())!= null) 
			{
				if (s.startsWith("Registered"))	// "Registered" variable found - update its value to "YES"
				{
					String msg = new String("Registered = YES ");
					out.write(msg,0,msg.length());
					out.newLine();	// add newline at the end
					continue;
				}
				else // remaining lines are copied without any modifications
				{
					out.write(s,0,s.length());
					out.newLine();	// add newline at the end
				}
			} // end while s = readline
			out.flush();
			out.close();
			in.close();
			
			// *** delete the result file and rename temporary file as result file
			
			f		= new File(fName);
			File	tF	= new File(tmpFile);

			if (f.delete())		// delete the file
			{
				// file got deleted successfully
				tF.renameTo(f);	// rename the temporary file
			}
		}
		catch (Exception e)
		{	
			System.out.println(" Error in file IO: ConfigDelegate " + e );	
			return false;
		}
		return true;
	}// end setRegDone
}

