
/**.
 *	Each webpage may have objects embedded in it. 
 *	These objects need to brought concurrently alongwith main page. 
 *	This class takes care of fetching those components.
 *	Also, at a time only a certain maximum number of components can be brought concurrently. 
 *	This needs synchronization between the threads created to fetch those components. 
 *	This class takes care of all such things. 
 *
 */
 

import java.lang.*;
import java.util.*;

class FetchComponent extends Thread
{
	Thread 		tFetch;		// current thread
	ArrayList	listImg;	// list of component URL including relative URLs
	int 		numLastCmp;	// number of elements in listImg before it is modified
	ArrayList 	completeURL;	// list of absolute component URLs 
	PageDetails	PD;		
	PageResult	PR;
	String 		curDir;		// Current directory of the URL of the page
	boolean 	doneCmp;	// flag to indicate that all the URLs of components on the page are received
	int 		threadSch;	// total number of thread scheduled 	
	boolean 	waitNewImg;	// scheduler  waiting for new image URL to arrive
	
	ComponentDetails	CD;	
	ComponentResult  	CR;	

	ArrayList		CDList; // component details list
	ArrayList		CRList; // component results list

	NumThread 		ob1;	// numthred for controlling number of threads to schedule


	FetchComponent(ArrayList list, PageDetails PD, PageResult PR)
	{
		tFetch = new Thread(this);	
		tFetch.setName("FetchComponent");
		listImg = list;	
		completeURL = new ArrayList();
		this.PD = PD;
		this.PR = PR;
		//curDir = new String(getCurDir(PD.URL));

		waitNewImg = false;
		doneCmp    = false; 
		threadSch  = 0;

		CDList = new ArrayList();
		CRList = new ArrayList();
			
	 	ob1 = new NumThread();	
		tFetch.start();
	}
	
	public void run()
	{
		// run till all components are fetched 
		while ((!doneCmp) || (threadSch < completeURL.size()))
		{
		 	f();
		} // end while

		ob1.exitPage();

	} // end run

	synchronized void f()
	{
			if ((threadSch == completeURL.size()) && !doneCmp)
			{
				// all components in current list are brought but page fetch has not been completed.
				try
				{
					waitNewImg =  true;
					wait();		// wait for event: New URL added to the list of components
				}
				catch (InterruptedException e)
				{
					System.out.println("Interrupted Exception Caught ");
				} 
				return;
			}  // end if
			while (threadSch < completeURL.size())	// there are components yet to be fetched 
			{
				if (ob1.curVal() == 0)	// exhausted all the active threads
				{
					ob1.waitAnyExit(true); // set the flag to indicate waiting for any thread to exit
					ob1.waitExitThread();
				}

				// schedule new thread 
				String cmpURL = (String) completeURL.get(threadSch);	// get the URL of the compoenent

				CDList.add(new ComponentDetails(cmpURL, PD.Port));
				CRList.add(new ComponentResult());
				PR.compRes.add(CRList.get(threadSch));	// add the component result object to page results
				ob1.decrease();		// decrease number of threads that can be created	
				Component tt = new Component((ComponentDetails) CDList.get(threadSch), (ComponentResult) CRList.get(threadSch), ob1);
				threadSch++;
			}
	}

	/**.
	 *	The page is searched for embedded componets using information about tags. 
	 *	All the URLs retrieved might be mix of absolute and relative URLs. 
	 *	All relative URLs are converted to absolute URL. Then unique URLs are identified.
	 *	All such unique URLs have to brought. 
	 */

	synchronized public void getNewImg()
	{
		int	size1;	// sizes of listImg list 
		int 	ind=0;	// index variable
		size1 = listImg.size();
			
		curDir = new String(getCurDir(PD.URL));
		
		while ( numLastCmp < size1)
		{
			String completeImg = new String("");	// complete url of the component
			String pic = new String("");
			pic = (String) listImg.get(numLastCmp);

			if (pic.startsWith("http://") == false) 
			{
				if ((pic.startsWith("../") == false)&& (pic.startsWith("./")==false))	
				{
					if (pic.startsWith("/") == true) // references with current directory, ignore
						completeImg = new String(getBaseDir(curDir) + pic.substring(1));
					else
						// there is no reference current/parent directory
						completeImg = new String(curDir + pic); 
				}
				else // process the string 
				{
					StringBuffer tmpPic = new StringBuffer(pic);	
					StringBuffer tmpCurDir = new StringBuffer(curDir);
				
					tmpCurDir.reverse();
					while(tmpPic.charAt(0) == '.')
					{
						if((tmpPic.charAt(1) == '.') && (tmpPic.charAt(2) == '/'))
   						{	
							tmpPic.delete(0,3);
							if (tmpCurDir.charAt(0) == '/')
								tmpCurDir.deleteCharAt(0);
							while (tmpCurDir.charAt(0) != '/')
								tmpCurDir.deleteCharAt(0);
						}
						else if (tmpPic.charAt(1) == '/')
						{		
							tmpPic.delete(0,2); 
						}
						else	break;
					} // end while
					tmpCurDir.reverse();
					completeImg = new String(tmpCurDir.toString() + tmpPic.toString());	
				}// end else
			} // end if
			else  // complete URL of the image is specified
			{
				completeImg = new String(pic);
			}
			synchronized (completeURL)
			{
				// check whether the image is already in the list	
				int compind=0;   // Component index - temporary variable
				compind = completeURL.indexOf(completeImg);
				if ( compind < 0)   // compind == -1  implies the image is not in the list.
				{
					completeURL.add(completeImg);
				}
			}// end sync
			numLastCmp += 1;
		} // end while
		if (waitNewImg)		
		{
			waitNewImg = false;
			notify();
		}
	} // end get

	/**.
	 *	Given an URL, getCurDir gets current directory for that page. 
	 *	This is needed when complete URL for images is to be found and images are referred with current directory.
	 *	Current directory for an URL is simply the string remaining after removing last component. 
	 *	eg. current directory for "http://www.google.com/images/logo.gif" is  "http://www.google.com/images/"
	 */

	private String getCurDir(String PgURL)
	{
		int 	tempi;	// temporary index variable
		String  tmpStr = new String(""); // temporary string variable

		//get the current directory
		tempi = PgURL.lastIndexOf('/');	
		tmpStr = PgURL.substring(0,tempi+1);
		return tmpStr;
	}


	/**.
	 *	getBaseDir returns the Base dirctory (fully qualified domain name) for given URL.
	 *	eg. Base Directory for  http://www.google.com/images/logo.gif is "http://www.google.com/"
	 *
	 */
	private String getBaseDir(String PgURL)
	{
		int 	tempi = 0;	// temporary index variable
		String  tmpStr = new String(""); // temporary string variable

		
		// find index of third '/' (first two '/' will be after http:)
		// the start "http://" ignored by searching for '/' after 8th char
		tempi = PgURL.indexOf('/',8);	
		tmpStr = PgURL.substring(0,tempi+1);
		return tmpStr;
	}

	/**.
	 *	 notify that page fetching is complete
	 */
	synchronized void done()
	{
		doneCmp = true;
		
		if (waitNewImg)		// if thread is waiting for new URLs, notify it
		{
			waitNewImg = false;
			notify();
		}
	}
} 

