import java.io.*;
import java.util.*;
import com.quiotix.html.parser.*;
import com.quiotix.html.parser.HtmlDocument.Attribute;
import com.quiotix.html.parser.HtmlDocument.AttributeList;

class GetImgTag extends Thread
{
	Thread 		t;		// current thread
	PageDetails 	PD;		// Details of the page
	PageResult 	PR;		// Results of the page
	String		Fname;		// the name of the file being fetched. a local file with same name will be created 
	File		f;		// File object for the above mentioned file
	boolean		usingFile;	// flag to indicate whether file to be written is in use
	boolean		pageModified;	// whether file has been modified from last access
	boolean 	pageFetchDone;	// flag to indicate the completion of page fetch
	ArrayList	ImgNameList;	// array list holding name of images in the page
	int 		numLast=0;	// number of elements in the list before list was modified
	FetchComponent 	fCmp;		

	/* HTML document is written as and when data is available. 
	 * Each time data is written to file, file is searched for the URLs of embedded images. 
	 * This leads to redundant work which can be avoided if the file is read from the position it was analyzed last time
	 * But all the data cannot be skipped directly because there might be some broken tags while last reading. 
	 * To accomodate those, the file is actully read from slightly back position.
	 *
	 * numAnalyzed stores the offset in the file till where it is analyzed for image and other tags. 
	 * 
	 * Currently, not implemented. 
	 */
	// long		numAnalyzed = 0;

	final String	BASE_DIR_LOG	= new String("/usr/local/webtest/logs/");

	GetImgTag(PageDetails PgD, PageResult PgR)
	{
		t = new Thread(this);
		t.setName("GetImgTag");
		PD = PgD;
		PR = PgR;

		int     ind; // a local variable used below
		
		/*	Find the file name from the URL
		 *	The URL can be in either of following three format
		 *	1. http://www.google.com
		 *	2. http://www.google.com/ (notice the '/' at the end)
		 *	3. http://www.google.com/index.html
		 *
		 *	in first two case file name will be empty string 
		 *	and in third case, file name is index.html
		 *
		 *	Then current date is added to the file name retrieved above. 
		 *	This is to ensure the uniqueness of filename
		 *	The procedure of addind date to the file name is explained ahead.
		 *
		 */

		ind = PD.URL.lastIndexOf('/');

		// case 2
		if ((ind+1) == PD.URL.length()) 
		{
			Fname = "";
		}
		
		// case 1
	 	else if (numSlash(PD.URL) < 3)
		{
			Fname = "";
		}

		// case 3
		else
		{
			Fname = new String(PD.URL.substring(ind+1));
		}
		/*  add the current date at the begining of the file name
		 *  here date means the a number representing that date
		 *  this number is equivalent of the number of milli seconds from 1st Jan 1970 to the current date/time.
		 */
		Date D = new Date();
		long L = D.getTime();
		Fname = BASE_DIR_LOG + L + Fname;

		// System.out.println("GetImgTag: File name " + Fname);		

		// create new  file with name Fname
		f = new File(Fname);
		try
		{
			// System.out.println("GetImgTag:Creating File " + Fname);	
			f.createNewFile();
		}
		catch (Exception e)
		{
			System.out.println(" Error in creating file  " + e );
		}
		
		System.out.println("GetImgTag: Setting file name "+ Fname);
		PR.setFname(Fname);	// update the name of the file in result object

		usingFile = false;
		pageFetchDone = false;
		pageModified = false;
		ImgNameList = new ArrayList();

		fCmp = new FetchComponent(ImgNameList, PD, PR);	// create new thread for fetching the components 
		t.start();
	}

	public void run()
	{
		while(!pageFetchDone || pageModified)   // execute till page fetch not done or  page is modified
		{
			readtags();
		}

		if (pageFetchDone)
		{
			fCmp.done(); 
		}
		// wait for FetchComponent to finish
		try
		{
			fCmp.tFetch.join();
		}
		catch (InterruptedException e)
		{	
			System.out.println("GetImgTag	: Thread Interrupted");
			fCmp.tFetch.interrupt();
			return;
		}
	} // end run

	int numSlash(String url)
	{
		int numS=0;
		for (int i=0; i < url.length();i++)
		if (url.charAt(i) == '/')
			numS +=1;
		return numS;
	}

	synchronized void readtags()
	{
		//System.out.println("GetImgTag: in read tags ");
		if (!pageModified || usingFile)
		try 
		{
		
		/*	System.out.println("GetImgTag: Waiting for images ");
			System.out.println("GetImgTag: pageModified " + pageModified );			
			System.out.println("GetImgTag: usingFile " + usingFile );
		*/

			wait();

		/*	System.out.println("GetImgTag: Waiting for images .. Done");
			System.out.println("GetImgTag: pageModified " + pageModified );			
			System.out.println("GetImgTag: usingFile " + usingFile );
		*/

		}
		catch (InterruptedException e)
		{
			System.out.println("Interrupted Exception Caught ");
		}
		if (!pageModified)
		{
			System.out.println("Page not modified - Returning to run ");
			return; 
		}

		usingFile = true;	


		// find the images in the webpage
		// read the file 
		// get img tags
		// add to a list			

		try 
		{
			FileInputStream in = new FileInputStream(Fname);
			HtmlDocument document;

			document = new HtmlParser((InputStream)in).HtmlDocument();
			document.accept(new GetImgName(System.out, ImgNameList));
			in.close();
		}
		catch (Exception e)
		{
			System.out.println(" Exception in opening file Exception " + e);
		}

		usingFile = false;		// release the lock on file
		pageModified = false; 		// update the flag indicating file has been modified
		notify();			// notify for the "write" of file


		// if new componenets are added, then submit them to FetchComponent for further proecessing
		if (numLast < ImgNameList.size())
			fCmp.getNewImg();
		numLast = ImgNameList.size();
	}

	/** writing to file
	 *  arr is byte array
	 *  numBytes number of bytes from startOff in the array are written to the file
	 */ 	
	synchronized void write(byte arr[], int startOff, int numBytes)
	{
		// System.out.println("GetImgTag :" + PD.URL);
		// System.out.println("GetImgTag : inside File write ");

		if (usingFile)
		try
		{
			wait();
		}
		catch (InterruptedException e)
		{
			System.out.println("Interrupted Exception Caught ");
		}
		usingFile = true;
		//append the bytes in arr to file	
		try
		{	
			FileWriter out = new FileWriter(Fname, true); // parameter is file name and true is for append
			out.write(new String(arr, startOff, numBytes), 0, numBytes);
			out.flush();
			out.close();
		}
		catch (Exception e)
		{
			System.out.println(" exception in writing page to file : GetImgTag.java " + e);	
		}

		pageModified = true;
		usingFile = false;
		notify();
	}
	
	synchronized void pageComplete()
	{
		pageFetchDone = true;
		notify();	// notify to say page fetch is complete.
	}
}

