
/***************************************.
 * The main program for the test Delegate
 * If the Delegate is not registered, Registration process is completed.
 * The user input is read, if necessary.
 * After the registration is done, other threads such as SendAlive, Scheduler and UploadResult are started.
 *
 */

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

public class MyMain
{
	public static void main(String args[])
	{
		// flag to indicate whether registration is done. 
		// delegate operation is not started until registration is complete.
		boolean 	regDone = false;

		// configure various parameters for Delegate
		ConfigDelegate c = new ConfigDelegate();

		while (!regDone)
		{
			// === check whether user info file exist
			// if not, display the form and read the info from user and register the delegate

			final   String BASE_DIR_CONF = new String("/usr/local/webtest/conf/");
			final String  userInfoFile = new String(BASE_DIR_CONF + "userinfo.conf");

			File usrfl = new File(userInfoFile);
			if (!usrfl.exists()) 
			{
				// if userinfo file does not exist, create a form and get info
				Form	f = new	Form();
				f.waitForInfo();

				c.userInfo(f.name,f.email,f.region,f.city,f.link,f.os, f.priv);
				if (f.usePr)
				{
					// proxy is being used
					// call appropriate constructor with parameters
					ConfigProxy cpr	= new ConfigProxy("YES", f.proxyURL, f.proxyPort, f.proxyUsr, f.proxyPass, f.noProxy);
				}
				else // proxy is not used
				{
					ConfigProxy cpr = new ConfigProxy("NO");	
				}
				f.dispose();
				f	= null;
			} // end if user info file exists

			// calling this function ensures delegate registration
			// if delegate is not registered for some reason, sleep for some time and then again try to register
			regDone = c.regDel();
			if (!regDone)
			{
				try
				{
					Thread.sleep(600000); // sleep for 600 seconds before trying again for registration
				}
				catch (InterruptedException e)
				{

				}
			} // end if not regdone
		}// end while not regDone

		// *** Wait for some time before initializing scheduler*** //
		try
		{
			Thread.sleep(60000); // sleep for 60 seconds before initializing scheduler
		}
		catch (InterruptedException e)
		{

		}
	
		// starting scheduler 
		Scheduler	sch = new Scheduler(); 

		// Starting Upload Result thread
		UploadResult u = new UploadResult();

		// Start thread to send Alive message
		SendAlive 	sa = new SendAlive();
	}
}
