
/**.
 *	This class is used to bring a page located at URL. 
 *	The details of page request are specified with an object of PageDetails.
 *	A connection is established to the server and request is sent. 
 *	In response, the server sends its response. The header for the response is seperated and analyzed. 
 *	The various results for the page are collected.
 *	
 */

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

class Page extends Thread implements HttpHeadGlobal
{
	Thread 			t;	
	PageDetails		PgLocal;	
	PageResult		PRLocal;
	ComponentDetails	CmpDetail;
	ComponentResult		CmpResult;
	

	Page(PageDetails PD, PageResult PR)
	{
		t = new Thread(this);
		t.setName("Page");
		PgLocal = PD;	
		PRLocal = PR;
		t.start();
	}

	public void run()
	{
		/* algorithm 
		* connect to the server 
		* get the page
		* get the list of components
		* for each component create a seperate thread  and fetch 
		*/
		String		Request = new String("");		// request sent to the server/proxy
		Socket 		s;			// socket descriptor
		String		pageStr=new String("");	// html page stored as string - using StringBuffer for modifications.
		String		fileName= new String(""); // name of the file brought
		InputStream	rd;
		OutputStream 	wr;
		int 		n; 
		final int	MAX_SIZE = 4096;		
		byte 		buff[]= new byte[MAX_SIZE]; // buffer to read the data from socket  

		long 		T1, T2, T3, T4, T5, T6;		// Timestamps 

		int		condel=0;	// Connection delay - start with 0
		int 		sndreq=0;	// Send request time - start with 0
		int		fb=0;		// First byte latency - start with 0
		int		pgDL=0;		// download time for only the HTML code of page
		int		cmpPgDL=0;	// download time for complete page including all the components

		Date		D1;

		// flag to indiacate whether current url redirects - remains true when respose code is 3XX
		boolean		redir = true;		

		boolean		HeaderDone = false;  		// flag to indicate whether header is removed
		int		pgCodeSize = 0;			// Size of the HTML code for the given URL
		int		headSize = 0;			// http header size
		String		PgHeader = new String(""); 	// page header stored as string
		
		GetImgTag	Itag;				// extracting img tags from html source

		Itag = null;

		D1 = new Date();
		T1 = D1.getTime();

		while(redir == true)
		{
		Request = new String(ConstructReq());		// construct the http request for fetching the page

		PRLocal.PgURL = PgLocal.URL; 	// URL of the page

		// open the connection to server or proxy (depending on configuration)
		OwnSocket S = new OwnSocket(PgLocal.URL, PgLocal.Port);
		if (S.isErr()) // check if there is any error in creating socket
		{
			PRLocal.setFailure(S.getErr());
			return;
		}

		// note down the connection delay
		D1 = new Date();
		T2 = D1.getTime();
		condel = (int) (T2-T1);

		try
		{
			rd = S.s.getInputStream();
			wr = S.s.getOutputStream();
			wr.write(Request.getBytes());  // write the http request to socket

			D1 = new Date();
			T3 = D1.getTime();
			sndreq = (int) (T3 - T2); // "send request"  delay

			boolean	HeadRemoved = false;	// flag indicating header has been removed 
							// before writing html code to file
			pageStr = new String("");
			HeaderDone = false;

			
			// flag for measuring first byte latency. 
			// once the latency is measured (i.e. after first read()), flag is set to 1
			int flagfirst = 0;

			while ((n=rd.read(buff,0,MAX_SIZE)) >= 0)    // start reading the data
			{
				// printing the response from server
				// for (int i=0;i<n;i++)
				// System.out.print((char)buff[i]);
				if (flagfirst == 0)	
				{
					// first byte received, note the timing
					flagfirst = 1;
					D1 = new Date();
					T4  = D1.getTime();
					fb = (int) (T4 - T3);
				}
				if (!HeaderDone)
				{	
					pageStr +=  new String(buff,0,n);
					// get HTTP header for the page
					PgHeader = getHeader(pageStr);	

					// if PgHeader is empty string, Header has not received completely yet.
					if (PgHeader.equals("") == false) 
					{
						HeaderDone = true;
						headSize  += PgHeader.length();
					}
					/* Analysis of HTTP Header */
					if (HeaderDone)
					{

						//System.out.println("*** Analyzing header... ");
						String	statLine = new String("");	// Status Line - contains response code	
						String	respCode = new String("");	// HTTP Response code
						int	conLen;				// Length of the response body (in bytes)
						String	conType = new String("");	// Response Content-type
						String	cookies = new String("");	// Cookies set
						
						int 	anal1=0, anal2=0; 		// index variable for analysis
						anal1 = PgHeader.indexOf(Delim);
						

						
						statLine = PgHeader.substring(0,anal1);	// get the status line
						anal1 = statLine.indexOf(" "); 		
						anal2 = statLine.indexOf(" ", anal1+1);


						respCode = statLine.substring(anal1+1, anal2);

						// Response code 4XX indicates client error
						if (respCode.startsWith("4"))
						{
							PRLocal.PgURL = PgLocal.URL; 	// URL of the page
							PRLocal.setFailure(respCode);	// set Response code received as error message 
							redir = false;	// no need of this 
							return;
						}

						// Response code 5XX indicates client error
						if (respCode.startsWith("5"))
						{
							PRLocal.PgURL = PgLocal.URL; 	// URL of the page
							PRLocal.setFailure(respCode);	// set Response code received as error message 
							redir = false;
						}

						if (respCode.equals("200"))
						{
							Itag = new GetImgTag(PgLocal, PRLocal); 
							redir = false;
						}

						if (respCode.startsWith("3") ==  true )	// redirection 
						{
							String	newURL = new String("");

							newURL = getHeadParam(PgHeader,"Location"); 

							// Fetch the new URL
							// assume that location gives absolute URL 
							PgLocal.URL = newURL;	
							S.s.close();		// close currently open socket
							break;
						}

						// FUTURE ENHANCEMENTS - add code to get cookie


					}	// end of analysing header - if HeaderDone
				} // end of removing header - if header not done
				if (HeaderDone)
				{
					if (HeadRemoved)
					{
						Itag.write(buff,0,n);	// write the data received
						pgCodeSize += n;	// size of the page incremented
					}
					else
					{
						int 	ind = PgHeader.length();
						// write data without header
						Itag.write(pageStr.getBytes(), ind, pageStr.length() - ind);
						HeadRemoved = true;
						pgCodeSize +=  pageStr.length() - ind;	
					}
				} // end if headerdone
			} //end while

			// if page has not yet been brought, pageComplete method cannot be invoked
			if (!redir)
				Itag.pageComplete();
			S.s.close();
		}
		catch (Exception e)
		{
		// 	System.out.println("Exception in getting page "+ e);

			/**. Get error message correctly. 
			 *	The exception string is
			 *		+ The name of the actual class of this object 
			 *		+ ": " (a colon and a space) 
			 *		+ The result of the getMessage() method for this object 
			 *	We just want the name of the class. Remaining components are removed.
			 *	Also, at the beginning of each class, the hierarchy of the classes is presented. 
			 *	The hierarchy is eliminated and only final class name is retrieved. 
			 *	Eg. If the message is "java.net.UnknownHostException: vsnlprox.iitk.ac.in", 
			 *		only UnknownHostException is retrieved.
			 */


			String err      = new String();
			err             = e.toString();

			String str2     = err.substring(0, err.indexOf(':'));	// remove the colon and further message

			String str3     = str2.substring(str2.lastIndexOf('.')+1); // remove hierarchy except last component

			PRLocal.setFailure(str3);	// set final error message

			return;	
		} // end try

		}	// end while redir

		// this time stamp will record the time required to fetch the code of page
		D1 = new Date();
		T5 = D1.getTime();	
		pgDL = (int) (T5 -T1);

		// wait for img tag to finish
		try
		{
			if (Itag.t.isAlive())
				Itag.t.join();
		}
		catch (InterruptedException e)
		{	
			System.out.println("Page	: Thread Interrupted");
				
			// Thread is interrupted on time out. 
			// hence pack up all the data and return
			// Error will be set by higher level object
			D1 = new Date();
			T6 = D1.getTime();
			cmpPgDL = (int) (T6 - T1);
		
		/* 	The HTML code of the page is also considered as the component of the page. 
		 *	This is done in the view that the actual page is composed of many components and original page is 
		 *	just a  part of complete page.
		 */	
			ComponentResult	pgCmpRes = new ComponentResult();  
			
			pgCmpRes.cmpURL 	= PgLocal.URL; 		// URL is same as that of page
			pgCmpRes.contentType 	= new String("TXT"); 	// Content of the document is text/html
			pgCmpRes.connectDelay 	= condel;
			pgCmpRes.sendReqDelay 	= sndreq;
			pgCmpRes.FBLatency 	= fb;
			pgCmpRes.cmpDL		= pgDL;
			pgCmpRes.cSize		= pgCodeSize; // pageStr.length(); 
			pgCmpRes.headerSize	= headSize ;
		
			PRLocal.compRes.add(pgCmpRes);	// add this component to the list

			PRLocal.PgURL = PgLocal.URL; 
			PRLocal.CompleteDL = (int) (T6-T1); 
			PRLocal.setSize(); 	// set  "complete size of the page" and "number of components" in result
			return;
		}
		
		// this timestamp will give the total time required to fetch the page alongwith all the components
		D1 = new Date();
		T6 = D1.getTime();
		cmpPgDL = (int) (T6 - T1);
	
	
	/* 	The HTML code of the page is also considered as the component of the page. 
	 *	This is done in the view that the actual page is composed of many components and original page is 
	 *	just a  part of complete page.
	 */	
		ComponentResult	pgCmpRes = new ComponentResult();  
		
		pgCmpRes.cmpURL 	= PgLocal.URL; 		// URL is same as that of page
		pgCmpRes.contentType 	= new String("TXT"); 	// Content of the document is text/html
		pgCmpRes.connectDelay 	= condel;
		pgCmpRes.sendReqDelay 	= sndreq;
		pgCmpRes.FBLatency 	= fb;
		pgCmpRes.cmpDL		= pgDL;
		pgCmpRes.cSize		= pgCodeSize; // pageStr.length(); 
		pgCmpRes.headerSize	= headSize ;
	
		PRLocal.compRes.add(pgCmpRes);	// add this component to the list

		PRLocal.PgURL = PgLocal.URL; 
		PRLocal.CompleteDL = (int) (T6-T1); 
		PRLocal.setSize(); 	// set  "complete size of the page" and "number of components" in result
	}// end run

/**
 * 	Create the URL in case proxy is present
 *	The request created is in the format  http://www.somesite.com:<port>/path/to/page.html
 * 
 */

	private String ProxyReq(String tmpURL, int port)
	{
		int	index = 0;	
		String	finStr = new String("");
		index = tmpURL.indexOf('/',8);  // the URL starts with "http://", hence it is ingnored
		

		finStr = tmpURL.substring(0, index);

		finStr += ":" + port;
		finStr += tmpURL.substring(index);

		return finStr;
	}

	private String ConstructReq()
	{
		String 	Tmp = new String("");		// String holding temporary request	
		String 	Httpv = new String("");		// HTTP Version for the page

		boolean	proxyStatus = Global.proxyStatus(PgLocal.URL);	// flag for use of proxy

		// Read the Request Method
		if (PgLocal.Method.equalsIgnoreCase("Get") == true )
			Tmp = GET;
		if (PgLocal.Method.equalsIgnoreCase("Post") == true )
			Tmp = POST;

		if (proxyStatus == true)  // proxy is being used then provide the port number with URL
		{	Tmp += ProxyReq(PgLocal.URL, PgLocal.Port) ; }
		else
		{	Tmp += PgLocal.URL; }
	
		// add the parameters if method is GET	
		if (PgLocal.Method.equalsIgnoreCase("Get") == true )
		{
			if (PgLocal.PgParam.length() > 0) // parameters have to be sent with request
			{
				Tmp += "?";
				Tmp += paramReq(PgLocal.PgParam, PgLocal.DataSet); 
			}
		}

		// add http version and delimiter
		// Read the Http Version used to bring the page
		if (PgLocal.HTTPV.equalsIgnoreCase("1.0") == true )
			Httpv = HTTP10;

		// future support for HTTP Version 1.1 
		//if (PgLocal.HTTPV.equalsIgnoreCase("1.1") == true )   
			//Httpv = HTTP11;

		Tmp += Httpv + Delim; 

		if (proxyStatus == true) 
		{

			// proxy is being used - provide authentication
			// use proxy authorization only if the authorization string is present. 
			if (Global.authUser.length() > 0) 
				Tmp += ProxyAuth + Global.authUser + Delim; 

			// provide "pragma" header to make sure that proxy does not serve from cache
			Tmp += Pragma + Delim;
		}

		// add the parameters if method is POST 
		if (PgLocal.Method.equalsIgnoreCase("Post") == true )
		{
			String PostDat = new String (paramReq(PgLocal.PgParam, PgLocal.DataSet)); 
		
			Tmp += contType + urlEncode + Delim ;		// Add Content-Type field to request
			Tmp += contLen + PostDat.length() + Delim;	// Add Content-Length field to request
			Tmp += Delim ;
			Tmp += PostDat;
		}

		Tmp +=  Delim;

		return Tmp;
	}

/* For some GET requests and for all POST request, parameters have to be sent alongwith the values. 
 * this method constructs the request in the form of parameter-value pair as per the requirement specified in RFC[1738].
 * 
 */
	private String paramReq(String Parameter, String Value)
	{
		String 	tmp = new String("");
		String 	tParam = new String("");
		String 	tVal = new String("");
		int 	indP1, indP2;	// index in the string of Parameter
		int	indV1, indV2;	// index in the string of Value 
		int 	numParam=0;	// number of parameters in the url
		
		indP2=0;
		indV2=0;

		while(true)
		{
			indP1 = Parameter.indexOf((int) paramDel ,indP2);	
			if (indP1 < 0)  // all parameters finished
				break;
			indV1 = Value.indexOf((int) paramDel ,indV2);	
			if (numParam > 0)
				tmp += '&';
				

			tParam = removeSpecial(Parameter.substring(indP2, indP1));	

			tVal = removeSpecial(Value.substring(indV2, indV1));	

	
			tmp += tParam + "=" + tVal; 
	
			indP2 = indP1 + 1;
			indV2 = indV1 + 1;
			numParam++;

		}
		return tmp;
	}	

/**.
 *	Certain characters cannot be sent "as it is" to the server.
 *	Instead, a % character followed by hex encoding of the character is sent to the server. 
 * 	All such characters are removed from the server and replaced with encoding. 
 */
	private String removeSpecial(String withSpecialChar)
	{	
		String	tmpR = new String("");

		// the reserverd and unsafe characters as specified in RFC 1738
		// as per RFC 1738, space should be transfered encoded as %20, but mozilla/konqueror transfer a space as + 
		// here the encoding is done as per the implementation of browser
		String	ResUnsafeChar = new String("$&+,/:;=?@\"<>#%{}|\\^~[]`\n");

		char hex[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'}; // Hex values 
	
		String inR = new String(withSpecialChar);
		//String	inR = new String(withSpecialChar.replace(' ', '+')); // replace all blanks by + 
		int 	ind=0;	
		
		// System.out.println(" input is " + inR);	

		while(ind < inR.length())
		{
			char ch =  inR.charAt(ind);
			byte chB = (byte) inR.charAt(ind);
			
			// if character is reserved or  unsafe change it to its hex representation
			if ( ResUnsafeChar.indexOf(ch) > 0)  
			{
				String r= new String("%" + hex[(chB >> 4) & 0x0f] + hex[chB & 0x0f]); // get the hex value
				tmpR += r;
			}
		 	else 
				tmpR += inR.charAt(ind);			
			ind++;

		}
		inR = tmpR.replace(' ', '+');
		return inR;	
	}

/** 
 *  getheader extracts out the http header of the page from the received data
 *  
 *  if header is completely brought, it is returned 
 *  if header is incomplete, an empty string is returned indicating "header not complete"
 *  
 */
	private String getHeader(String Src)
	{
		String 	HSrc = Src;		// local variable for parameter 1
		String 	HHead = new String("");	// local variable for parameter 2
		int	i;			// index variable 
		int	itemp =0, itemp2=0;	// temporary index variables
		String 	Hcomp = new String("");	// header component	
		
		while (true)
		{
			itemp = HSrc.indexOf(Delim,itemp2);
			if (itemp < 0) return HHead;
			Hcomp = HSrc.substring(itemp2,itemp);

			if (Hcomp.equals(""))
			{		
				HHead = new String(HSrc.substring(0, itemp+2));
				return HHead;
			}
			itemp2 = itemp + 2;
		}
	}

/* 	
 * getHeadParam returns the value for the particular parameter in HTTP header 
 * The input is complete Header string and the parameter whose value is to be retrieved
 */
	private String getHeadParam(String head, String paramVal)
	{
		String 	res = new String("");		// result string
		int anal1=0, anal2=0;			// index variables

		anal1 = head.indexOf(paramVal);
			
		if (anal1 < 0) // required parameter not found
		{
			return res; //	return empty string indicating failure
		} 

		anal1 =	head.indexOf(" ", anal1);
		anal2 = head.indexOf(Delim, anal1);

		if (anal2 < 0) // malformed header
		{
			return res; //	return empty string indicating failure
		} 
		res = head.substring(anal1+1, anal2);
		return res;
	}
}// end class Page	

