
/**.
 *	This class is for accessing the results of various transactions. 
 *	All the results are stored in a text file in a predefined format.
 *	The results for various transactions are submitted by Transaction class.	
 *	This class contains static methods. Hence they are invoked without specifying any object. 
 *	And there will only be one Result object which ensures there will not be any race coditions arising to access the results. 
 */

import java.io.FileWriter;
import java.util.*;
import java.io.*;


class Result
{
	int i=0;	// dummy variable

	Result()
	{

	}
	
	/* submitTr accepts the TransResult object. The contents are stored in the result file in a predefined format.
	 * If contents are stored sucessfully, TRUE is returned to indicate success.
	 * FALSE is returned to indicate failure to do so.
	 */ 

	synchronized static boolean submitTr(TransResult Tr)
	{
		String 		fName = new String(Global.getResultFileName());
		TransResult 	T = Tr;
		PageResult 	P;
		ComponentResult	C; 
		
		// if End-to-End Response time is less than 200 milliseconds 
		// but the transaction is not failed, indicates some kind of error. 
		// don't write such results to the file.
		if ((T.EtoE < 200) && (T.trSuccess))  
			return true;

		try
		{
			FileWriter	out = new FileWriter(fName, true);	// parameter is file name and "true" is for append 

			// code to write transaction result 
			// fill the details afterwards 
			out.write(new String("###JSTART\n"));
			out.write(new String("JID "+ T.JobID + "\n"));
			out.write(new String("STM "+ T.StartTime + "\n"));	// Start Time Of Transaction 
			out.write(new String("NUM "+ T.NumSteps + "\n"));	// Number of Steps
			out.write(new String("ETE "+ T.EtoE + "\n"));		// End-to-End Response time

			if (!T.trSuccess) // error occured in executing transaction
			{
				out.write(new String("ERR 1"+"\n" ));		// set error = 1 indicating failure
				out.write(new String("FSN "+T.failAt+"\n" ));	// write step number of failure
			}
			else	// transaction completed without any errors
			{
				out.write(new String("ERR 0"+"\n" ));	// set error = 0 indicating success
				out.write(new String("FSN 0"+"\n" ));	// step number 0 indicates success
			}

			out.write(new String("###JSTOP\n"));

			int i=0;
			while (i< T.ListPageRes.size())
			{
				P = (PageResult) T.ListPageRes.get(i);
	
				out.write(new String("##PSTART\n"));
				out.write(new String("URL " + P.PgURL+"\n" ));
				out.write(new String("SIZ "+ P.completeSize +"\n"  ));
				out.write(new String("DEL "+ P.CompleteDL +"\n" ));
				out.write(new String("CMP "+ P.NumComp +"\n" ));

				if (!P.pgSuccess) // error occured while fetching the page
				{
					out.write(new String("ERR 1"+"\n" ));		// set error = 1 indicating failure
					out.write(new String("FRS "+P.failReason+"\n" ));// write failure reason also
				}
				else	// page fetched without any errors
				{
					out.write(new String("ERR 0"+"\n" ));	// set error = 0 indicating success
					out.write(new String("FRS "+"\n" ));	// leave reason field blank
				}
				out.write(new String("##PEND\n" ));

				// print the results for all the components
				int I =0;
				out.write(new String("##CSTART\n"));
				while (I < P.compRes.size())
				{
					C = (ComponentResult) P.compRes.get(I);
					out.write(new String("URL " + C.cmpURL+"\n"));
					out.write(new String("CNT " + C.contentType  +"\n"));
					out.write(new String("DEL "+C.connectDelay+" "+C.sendReqDelay+" "+C.FBLatency+" "+C.cmpDL +"\n")); 
					out.write(new String("SIZ " + C.cSize +"\n"));
					out.write(new String("HDS " + C.headerSize+"\n"));

				// for a component ERR and FRS fields are given only if there is an error

					if (!C.cmpSuccess) // error occured while fetching the component
					{
					out.write(new String("ERR 1"+"\n" ));		// set error = 1 indicating failure
					out.write(new String("FRS "+C.failReason+"\n" ));// write failure reason also
					}

					out.write(new String("#"+"\n"));
					I++;	// get next component
				}
				out.write(new String("##CEND\n" ));
				i++;    // get next page 
			}
			out.write(new String("###JEND\n"));
			out.flush();
			out.close();
		}
		catch (Exception e)
		{
			System.out.println(" exception in writing page to file : Result.java " + e);
			return false;
		}

		return true;
	}// end method submitTr	


/**.
 *	When a HTML page is accessed, its contents are stored in a file.
 *	delPageFiles deletes all such pages once test is done.
 *	This is necessary so as to avoid unnecessary files being store on disk.
 *	THIS IS UNTESTED CODE
 */

	static void  delPageFiles(TransResult T)
	{

		TransResult 	TR = T;
		PageResult 	P;
		int i=0;

		while (i < T.ListPageRes.size())
		{
			P = (PageResult) T.ListPageRes.get(i);	// get page result object
			String 	delfName= P.pgFile;	// get the name of the file in which its contents are stored
			File	f	= new File(delfName);

			System.out.println("Result : Calling Delete File "+ delfName);

			try
			{
				if (f.delete())				// delete the file
				{
					// file got deleted successfully
					System.out.println("Result	: Deleted File " + delfName);
				}
			}
			catch (Exception e)
			{
				System.out.println("Result	: Error in Deleting the file " + e );	
			}
			i++;
		} // end while
		return;
	}

/*  getResult() will give result of single transaction monitored by Delegate. 
 *  The results are filled in object of FormatUploadRes.
 */

	synchronized static void getResult( FormatUploadRes ru )
	{
		FormatUploadRes r = ru;
		String		traR = new String("");	// result for complete transaction
		String		pgR  = new String("");	// result for a page - high level
		String		pgD  = new String("");	// detailed result for a page

		boolean 	jobdone = false;	// flag to indicate result of job have been read
		boolean 	pagedone = false;	// flag to indicate result of page have been read
		boolean 	compdone = false;	// flag to indicate result of components of page have been read

		// get the file name in which results are stored
		String 		fName = new String(Global.getResultFileName()); 

		boolean inside = false;	//flag to indicate it is currently inside a transaction result

		try 
		{
			BufferedReader input = new BufferedReader(new FileReader(fName));
			String s = new String();

			// first search for ###STA mark, which indicates there is a transaction result ahead
			while((s=input.readLine())!= null) 
			{
				if (s.startsWith("###JSTA"))
				{
					inside = true;
					break;
				}
			}
			// got a transaction result which is yet to be uploaded			
			
			while(((s=input.readLine())!= null) && inside) 
			{
				if (s.startsWith("###JEN"))
				{
					inside = false;
					continue;
				}
				if (s.startsWith("###JSTO"))
				{
					r.addT(traR);
					jobdone = true;	// results for the job have been read
					traR = "";
					continue;
				}
	
				if (s.startsWith("##PEN"))
				{
					r.addP(pgR);
					pagedone = true;
					pgR = "";
					continue;
				}
				if (s.startsWith("##CEN"))
				{
					r.addD(pgD);
					compdone = true;
					pgD = "";
					continue;
				}

				if (s.startsWith("##PSTA"))
				{
					pagedone = false;
					continue;
				}
				if (s.startsWith("##CSTA"))
				{
					compdone = false;
					continue;
				}
				if (jobdone == false) // job result getting
				{	
					// find jobid and number of steps
					if (s.startsWith("JID"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip the first token which contains "JID"
						r.JobID = st.nextToken();
						continue;
					}

					if (s.startsWith("NUM"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip the first token which contains "NUM"
						r.NumStep = Integer.parseInt(st.nextToken());
						continue;
					}
					if (s.startsWith ("STM"))
					{		
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip the first token which contains "NUM"
						// next token contain date and time
						// add date with a space at end
						String s1 = new String(st.nextToken() + " ");	
						s1 +=  st.nextToken();			// add time
						r.Stime = s1;
					}
					traR  += s + "\n";	// append the line at the end of currentline
					continue;
				}
				if (pagedone == false)
				{
					pgR   += s +"\n" ;
					continue;
				}
				if (compdone == false);
				{
					pgD += s +"\n";
					continue;
				}
			} // end while readline
			input.close();
	    	}
		catch(IOException e) 
		{
			System.out.println("Result : IOException "+e);
	    	}
	}
		

/* Whenever the result for a transaction is uploaded successfully at Reporting Centre, the Upload module invokes this function.
 * The parameter specifies the JobID and start time of the transaction.
 * The uploaded result is marked as "Uploaded". This makes sure that same result is not uploaded multiple times. 
 * All the uploaded results are purged using different method.
 * 
 */
	synchronized static void markUploaded(String JID, String Stime )
	{
		RandomAccessFile	f=null;		// stream for file read write 
		boolean 		done = false; 	// flag to indicate whether the result is marked 

		String			jid = new String(JID);
		String			stime = new String(Stime);	

		// following string variables are used to store the intermediate data read from file.
		// if all the strings match, then the file is updated. 
		String 			s1 = new String("");	// string read from a file
		String			s2 = new String("");	// string to store jobid
		String			s3 = new String("");	// string to store start time

		try
		{
			f = new RandomAccessFile( Global.getResultFileName() , "rw"); 
		}
		catch(IOException e) 
		{
			System.out.println(" Result :" + e );
		}

		try
		{
			int offset =0;	// maintain the offset of the current line in file
			while (((s1 = f.readLine()) != null) && !done)
			{
				if (s1.startsWith("###JSTA"))	// check if the string indicates start of transaction result
				{
					s2 = f.readLine();
					s3 = f.readLine();
					String cmpstr1 = new String("JID "+jid);
					String cmpstr2 = new String("STM "+stime);
	
					StringTokenizer st1 = new StringTokenizer(s2);
					st1.nextToken();	// skip the first token which contains "JID"
					if (jid.equalsIgnoreCase(st1.nextToken())) // try to match JobID 
					{	// JobID matched. now match date and time

						StringTokenizer st2 = new StringTokenizer(stime);
						StringTokenizer st3 = new StringTokenizer(s3);
						
						st3.nextToken(); // skip first toke which contains "STM"
						
						String 	tmpd = st3.nextToken();		// get date 
						String	tmpt = st3.nextToken();		// get time
						String	tmpd2 = st2.nextToken();	// get date 
						String 	tmpt2 = st2.nextToken();	// get time

						if (tmpd.equalsIgnoreCase(tmpd2) && tmpt.equalsIgnoreCase(tmpt2))
						{
							done = true; // JobID, Date and Time are matched.

							// get the offset of the ###JSTART which we have preserved
							f.seek(offset); 

							// replace ###JSTART by ###JUPLD# followed by a newline character
							// note that 9 characters are replaced by 9 characters. 
							// The new line has been removed while reading the line.
							// hence it again needs to be written. 
							f.writeBytes("###JUPLD#\n"); 
						}
					} // end if jid equals
				} // end if s1.startwith ##JSTA

				/*  increase the offset by 1 because at the end of every line, 
				 *  newline character is present which is not part of string s1.  
				 */
				offset += s1.length() + 1;
			}// end while s1 = f.readline
		} // end try
		catch(IOException e) 
		{
			System.out.println("Result : error" + e); 	
		}

		try
		{
			f.close();
		}
		catch(IOException e) 
		{
			System.out.println("Result : Unable to close the file " + e);
		}
	} // end markUploaded

/**  
 *  This method deletes all the entries of results which are already uploaded.
 *
 */
	synchronized static void purgeUploaded()
	{
		String	fName = new String("");	
		String 	tmpFile	= new String("resfile.tmp");

		fName = Global.getResultFileName();	// get result file name

		// create temporary file for storing result 
		File f = new File(tmpFile);
		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : Result.java " + e );
			}
		}
	
		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(tmpFile));
			BufferedReader in = new BufferedReader(new FileReader(fName));

			String 	s	= new String("");
			boolean inside	= false;	// flag to indicate whether current line is inside uploaded result

			// ***	copy contents of result file in temp file except the uploaded result
			while((s=in.readLine())!= null) 
			{
				if (s.startsWith("###JUP"))	// JUP indicates uploaded result
				{
					inside	= true;	// inside an uploaded result
					continue;
				}
				if (!inside)
				{
					out.write(s,0,s.length());
					out.newLine();	// add newline at the end
				}
				if (s.startsWith("###JEN"))	// JEN indicates end of a result
				{
					if (inside == true)
						inside = false;

				/*	if (inside)
					{
						inside = false;
						continue;
					}
				*/
				}
			}
			out.flush();
			out.close();
			in.close();
			
			// *** delete the result file and rename temporary file as result file
			
			f		= new File(fName);
			File	tF	= new File(tmpFile);

			if (f.delete())		// delete the file
			{
				// file got deleted successfully
				tF.renameTo(f);	// rename the temporary file
			}
		}
		catch (Exception e)
		{	
			System.out.println(" Error in file IO: Result.java " + e );	
		}
	}
}// end class

