
/**. 
 *   This class is to store the results of the transaction.
 *   The various parameters of the transaction are measured and stored in the object of this class. 
 *   The main result of transction is the End-to-End response time for complete transaction.
 *   Also, the detailed result of each step is stored in a list. Each component of this list is an object of class PageResult
 *   
 */

import java.util.*;

class TransResult
{
	String	JobID;		// JobID for the Transaction
	String	StartTime;	// Start time for this transaction - includes date and time
	int	NumSteps;	// Number of Steps in the transaction
	int	EtoE;		// End-to-End Response Time (in milli seconds)
	ArrayList ListPageRes;	// List of PageResult objects

	boolean	trSuccess;	// Flag to indicate whether transaction was successful - true indicates success
	int	failAt;		// If test has failed, then the step number at which failure is occured 

	/** The constructor accpets to arguments
	 *  JID is the JobID for the transaction and
	 *  ST is the start time of the transaction
	 */
	TransResult(String JID, String ST)
	{
		JobID 		= new String(JID);
		StartTime 	= new String(ST);
		ListPageRes	= new ArrayList();
		trSuccess 	= true; 	// start with success, in case of failure set it false
		failAt		= 0;		// initialized to zero to indicate success
	}
	public String getJID()
	{
		return JobID;
	}

	/**. 	If failure occurs, then this method is called with the step number at  which  failure occurred
	 *	
	 */
	public void setFailure(int stepNum)
	{
		trSuccess 	= false ; 	// failure occurred
		failAt		= stepNum;	// failure at "stepNum"
	}

	/**.
	 *	getAvg()  returns the combined result of two TransResult objects - current and the one in parameter
	 *	The resulting object is average of the various respose time parameters of the two results.  
	 *	If some object has error flag ON and the other is without error, in that case, the results of without error are taken
	 *	as the final result.
	 *	The assumption here is that both results - the current object and the one given in parameter - are of same job 
	 */
	 
	public TransResult getAvg(TransResult tr)
	{
		// Create a new result object. This object is initialized with JoBID and StartTime of current object. 
		// This is with an assumption that the two result objects do not differ much in their start time.
		
		TransResult avg = new TransResult(this.JobID, this.StartTime);
		
		//*** check the transaction error flag

		// both are failed,
		if ((!this.trSuccess) && (!tr.trSuccess))
		{
			//if both failed at same step, copy any of them directly 
			if (this.failAt == tr.failAt)
			{
				// copying all the pameters
				avg.NumSteps	= this.NumSteps;
				avg.EtoE	= this.EtoE;
				avg.ListPageRes = (ArrayList) this.ListPageRes.clone();
				avg.trSuccess	= this.trSuccess;
				avg.failAt	= this.failAt;

				return avg;
			}
			
			// if both are failed at different steps, choose the one which failed at later step  

			else if (this.failAt > tr.failAt)
			{
				// copying all the pameters
				avg.NumSteps	= this.NumSteps;
				avg.EtoE	= this.EtoE;
				avg.ListPageRes = (ArrayList) this.ListPageRes.clone();
				avg.trSuccess	= this.trSuccess;
				avg.failAt	= this.failAt;
				
				return avg;
			}
			else if (tr.failAt > this.failAt)
			{
				// copying all the pameters
				avg.NumSteps	= tr.NumSteps;
				avg.EtoE	= tr.EtoE;
				avg.ListPageRes =(ArrayList)  tr.ListPageRes.clone();
				avg.trSuccess	= tr.trSuccess;
				avg.failAt	= tr.failAt;
				return avg;
			}
		} // end if both failed
		
		// if only of them failed, then discard the failed results and copy the successful result directly

		// the result given in parameter is failed, hence discard it.
		if ((this.trSuccess) && (!tr.trSuccess))
		{
			avg.NumSteps	= this.NumSteps;
			avg.EtoE	= this.EtoE;
			avg.ListPageRes = (ArrayList) this.ListPageRes.clone();
			avg.trSuccess	= this.trSuccess;
			avg.failAt	= this.failAt;
			
			return avg;
		}

		// the current result is failed, hence discard it.
		if ((tr.trSuccess) && (!this.trSuccess))
		{
			avg.NumSteps	= tr.NumSteps;
			avg.EtoE	= tr.EtoE;
			avg.ListPageRes = (ArrayList) tr.ListPageRes.clone();
			avg.trSuccess	= tr.trSuccess;
			avg.failAt	= tr.failAt;
			
			return avg;
		}
		
		// end - error flag set for any of the result

		//*** Both the results are of successful completion
		
		// find the average of EtoE response time. Other parameters remain same.
		avg.NumSteps	= this.NumSteps;
		avg.EtoE	= (int)((this.EtoE + tr.EtoE)/2);
		avg.trSuccess	= this.trSuccess;
		avg.failAt	= this.failAt;

		//========== For each Page, calculate average of various parameters ==========
	
		for (int pInd=0; pInd < this.ListPageRes.size();pInd++)
		{
			PageResult	pThis	= new PageResult();
			PageResult	ptr	= new PageResult();
			PageResult	pAvg	= new PageResult();

			pThis	= (PageResult) this.ListPageRes.get(pInd);	
			ptr	= (PageResult) tr.ListPageRes.get(pInd);

			pAvg.PgURL	= pThis.PgURL;	// Page URL
			pAvg.completeSize = (int)((pThis.completeSize + ptr.completeSize)/2);	//Complete Page Size(including components)
			pAvg.CompleteDL	= (int)((pThis.CompleteDL + ptr.CompleteDL)/2);		// Download time for complete page.
			// no changes to pgSuccess and failReason attributes


			// copy the component result  array to temporary variable
			// the components are deleted from the temporary list as and when match is found

			ArrayList	tmpcTr	= new ArrayList();
			tmpcTr 			= (ArrayList) ptr.compRes.clone();	

			for (int cInd=0;cInd < pThis.compRes.size(); cInd++)
			{
				ComponentResult	cThis	= new ComponentResult();
				ComponentResult	ctr	= new ComponentResult();
				ComponentResult	cAvg	= new ComponentResult();
				cThis	= (ComponentResult) pThis.compRes.get(cInd);
			
				// === find the corresponding ComponentResult for this URL	
				boolean found = false ;
				for (int trInd = 0 ; !found && (trInd < tmpcTr.size()) ; trInd++)
				{
					ctr = (ComponentResult) tmpcTr.get(trInd);
					if (cThis.cmpURL.equals(ctr.cmpURL))
					{
						found = true;
					}
				}// end for trInd
			
				// two components having same URL are found
				// process them as earlier
				if (found)
				{
					cAvg.cmpURL	= cThis.cmpURL;
					cAvg.contentType = cThis.contentType;

					// check for failure
					
					// both failed
					if ((!cThis.cmpSuccess) && (!ctr.cmpSuccess))
					{
						cAvg.connectDelay	= cThis.connectDelay;
						cAvg.sendReqDelay	= cThis.sendReqDelay;
						cAvg.FBLatency		= cThis.FBLatency;
						cAvg.cmpDL		= cThis.cmpDL;
						cAvg.cSize		= cThis.cSize;
						cAvg.headerSize		= cThis.headerSize;
						cAvg.cmpSuccess		= cThis.cmpSuccess;
						cAvg.failReason		= cThis.failReason;
					}
					
					// the result given in parameter is failed, hence discard it
					else if((cThis.cmpSuccess) && (!ctr.cmpSuccess))
					{
						cAvg.connectDelay	= cThis.connectDelay;
						cAvg.sendReqDelay	= cThis.sendReqDelay;
						cAvg.FBLatency		= cThis.FBLatency;
						cAvg.cmpDL		= cThis.cmpDL;
						cAvg.cSize		= cThis.cSize;
						cAvg.headerSize		= cThis.headerSize;
						cAvg.cmpSuccess		= cThis.cmpSuccess;
						cAvg.failReason		= cThis.failReason;
					}

					// the current result is failed, hence discard it
					else if((!cThis.cmpSuccess) && (ctr.cmpSuccess))
					{
						cAvg.connectDelay	= ctr.connectDelay;
						cAvg.sendReqDelay	= ctr.sendReqDelay;
						cAvg.FBLatency		= ctr.FBLatency;
						cAvg.cmpDL		= ctr.cmpDL;
						cAvg.cSize		= ctr.cSize;
						cAvg.headerSize		= ctr.headerSize;
						cAvg.cmpSuccess		= ctr.cmpSuccess;
						cAvg.failReason		= ctr.failReason;
					}

					// both results are succesful, hence find the average of it.
					else 
					{
						cAvg.connectDelay	= (int) (cThis.connectDelay + ctr.connectDelay)/2;
						cAvg.sendReqDelay	= (int) (cThis.sendReqDelay + ctr.sendReqDelay)/2;
						cAvg.FBLatency		= (int) (cThis.FBLatency + ctr.FBLatency)/2;
						cAvg.cmpDL		= (int) (cThis.cmpDL + ctr.cmpDL)/2;
						cAvg.cSize		= (int) (cThis.cSize + ctr.cSize)/2;
						cAvg.headerSize		= (int) (cThis.headerSize + ctr.headerSize)/2;
						
						// cAvg.cmpSuccess		= cThis.cmpSuccess;
						// cAvg.failReason		= cThis.failReason;
					}
				}// end if found

				// current component not found in the list
				if (!found)
				{
					cAvg.connectDelay	= cThis.connectDelay;
					cAvg.sendReqDelay	= cThis.sendReqDelay;
					cAvg.FBLatency		= cThis.FBLatency;
					cAvg.cmpDL		= cThis.cmpDL;
					cAvg.cSize		= cThis.cSize;
					cAvg.headerSize		= cThis.headerSize;
					cAvg.cmpSuccess		= cThis.cmpSuccess;
					cAvg.failReason		= cThis.failReason;
				}

				// add the component result object
				if (pAvg.compRes.add(cAvg))
					; //do nothing to this

			}// end for cInd

			// set the parameters - number of components and total size of the page 
			pAvg.setSize();

			// add this page to the final transaction result 
			avg.ListPageRes.add(pAvg);
		
		}// end for pInd

		return avg;

	} // end getAvg

	/**.
	 * The HTML code download is stored in a file locally. 
	 * This method deletes all such files.
	 * delTrFile in turn calls to delPageFile method of PageResult
	 * 
	 */

	
	public boolean delTrFile()
	{
		int 		indp = 0; 	// index for page results

		// flag to indicate all the files have been deleted successfully
		// when it encounters an error from deleting page, it is set to false
		boolean 	success= true;	
		PageResult	P;

		while (indp < ListPageRes.size())
		{
			P = (PageResult) ListPageRes.get(indp);
			if (P.delPageFile() == false)
			{
				success = false;	
			}
			indp += 1;
		}
		return success;
	}	
}


