
/**.	This class does some boot-up work for delegate.
 *	Various parameters used to access Reporting Center are stored in a configuration file (rcparam.conf) 
 * 	[*** Actual location of rcparam.conf will depend on the operating system of installation]
 *	Intitially, this configuration file does not exist. 
 *	While the Delegate starts for the first time, the configuration file is created
 *	Once configuration file is created, this class does not have any function
 *	Any line starting with a '#' is considered to be comment and is ignored
 */

import java.io.File;
import java.io.FileWriter;
import java.io.*;
import java.util.*;

class ConfigDelegateUser
{
	final 	String BASE_DIR_CONF	= new String("C:/PerMoS/conf/");
	final String	configFile 	= new String(BASE_DIR_CONF + "rcparam.conf");
	final String	userInfoFile 	= new String(BASE_DIR_CONF + "userinfo.conf");

	ConfigDelegateUser()
	{
		//==== create  configuration file

		File f = new File(configFile);
	/*	if (f.exists() == true)	// if file exits, return
			return;
	*/

		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : ConfigDelegateUser.java " + e );
			}
			
			// File created....
			// Write Reporting Centre Parameters
			try
			{

				BufferedWriter 	out = new BufferedWriter(new FileWriter(configFile));
				
				out.write("### Please do NOT edit this file "); // warming message
				out.newLine();	
				out.write("ResFile = trans.res ");		// Name of the file for storing transaction results
				out.newLine();	// add newline at the end
				out.write("ProfileFile = monitor.pro ");		// Name of the file for storing various profiles
				out.newLine();	
				out.write("DelID =  "+ createDelID());		// Delegate ID
				out.newLine();

				// Parameters for uploading results for transactions
				out.write("UpResURL = http://www.yourserver.com/path/to/resupload.php ");	// URL for the page
				out.newLine();
				out.write("UpResPort = 80 ");	// Port number to be used to establish connection
				out.newLine();
				out.write("UpResMethod = POST ");	// HTTP Method to be used
				out.newLine();	
				out.write("UpResVers = 1.0 ");	// HTTP Version to be used
				out.newLine();

				// Parameters for sending alive message to the Reporting Centre
				out.write("AliveURL = http://www.yourserver.com/path/to/alive.php ");	// URL for the page
				out.newLine();
				out.write("AlivePort = 80 ");	// Port number to be used to establish connection
				out.newLine();
				out.write("AliveMethod = GET ");	// HTTP Method to be used
				out.newLine();	
				out.write("AliveVers = 1.0 ");	// HTTP Version to be used
				out.newLine();

				// Parameters for downloading the job profile
				out.write("JobDLURL = http://www.yourserver.com/path/to/getjob.php ");	// URL for the page
				out.newLine();
				out.write("JobDLPort = 80 ");	// Port number to be used to establish connection
				out.newLine();
				out.write("JobDLMethod = GET ");	// HTTP Method to be used
				out.newLine();	
				out.write("JobDLVers = 1.0 ");	// HTTP Version to be used
				out.newLine();

				// Parameters for Registration of Delegates
				out.write("RegURL = http://www.yourserver.com/path/to/register.php ");	// URL for the page
				out.newLine();
				out.write("RegPort = 80 ");	// Port number to be used to establish connection
				out.newLine();
				out.write("RegMethod = POST ");	// HTTP Method to be used
				out.newLine();	
				out.write("RegVers = 1.0 ");	// HTTP Version to be used
				out.newLine();

				out.flush();
				out.close();
			}
			catch (Exception e)
			{		
				System.out.println("ConfigDelegateUser : Unable to write to  rcparam.conf " );
			}
		}

	}

	/**.
	 *	This method creates an unique ID for this delegate and returns it.
	 *	DelID = Hex representation of the current time stamp 
	 */	

	private String createDelID()
	{
		Date 	d	= new Date();
		String id 	= Long.toHexString(d.getTime());

		return id;

	} // end createDelID

	/*	userInfo method creates a user info file and writes the information from the form.
	 *	This info is submitted to Reporting Centre while registration of Delegate 
	 */
	
	public void userInfo(String name, String email, String reg, String city, String inet, String os, String priv)
	{
		// === Create UserInfo file and store various information 
	
		File f = new File(userInfoFile);
		if (f.exists() == true)	// if file exits, return
			return;

		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : ConfigDelegateUser.java " + e );
			}
		}

		// file created, now update the parameters

		try
		{

			BufferedWriter 	out = new BufferedWriter(new FileWriter(userInfoFile));
			
			out.write("### Please do NOT edit this file "); // warming message
			out.newLine();	
			out.write("#Registered is a flag to indicate whether Delegate is Registered with Reporting Centre");
			out.newLine();

			// initial value is no indicating delegate is not registered
			// note the two spaces after "NO".
			// when Delegate is registered, it will be changed to "YES" with string of exact length
			out.write("Registered = NO  ");	
			
			out.newLine();
			out.write("Name = "+ name);		// name of the user
			out.newLine();
			out.write("Email = "+email);		// email address
			out.newLine();
			out.write("Region = "+reg);		// region of the delegate
			out.newLine();
			out.write("City = "+city);		// city of residence
			out.newLine();
			out.write("Link = "+inet);		// link to internet
			out.newLine();
			out.write("OS = "+os);			// operating system
			out.newLine();
			out.write("Priv = "+priv);		// privelege level for Delegate
			out.newLine();
			out.flush();
			out.close();
		}
		catch (Exception e)
		{		
			System.out.println("ConfigDelegateUser : Unable to write to rcparam.conf ");
		}
	}
}
