
/**.
 *      This class identifies the embedded objects of the page and lists the ULLs.
 *      This has been written with some modification to
 *      "Example visitor to dump out the links from an HTML document" by Brian Goetz, Quiotix
 *
 */

import java.util.*;
import java.io.*;
import com.quiotix.html.parser.*;
import com.quiotix.html.parser.HtmlDocument.Attribute;
import com.quiotix.html.parser.HtmlDocument.AttributeList;

public class GetImgName extends HtmlVisitor 
{

	protected 	PrintWriter out;
	public 		ArrayList Tmp;

	public GetImgName(OutputStream os)     { out = new PrintWriter(os); }

	public GetImgName(OutputStream os, String encoding)
		throws UnsupportedEncodingException {
		out = new PrintWriter( new OutputStreamWriter(os, encoding) );
	}

	public GetImgName(OutputStream os, ArrayList T)     { out = new PrintWriter(os); Tmp = T; }

	public GetImgName(OutputStream os, String encoding, ArrayList T)
		throws UnsupportedEncodingException 
	{
		out = new PrintWriter( new OutputStreamWriter(os, encoding) );
		Tmp = T;
	}
	

	public void finish() 
	{ out.flush(); }

	public void visit(HtmlDocument.Tag t) 
	{ 
		// get all the image tags
		if (t.tagName.equalsIgnoreCase("img")) 
		{
			for (Iterator i=t.attributeList.attributes.iterator(); i.hasNext(); ) 
			{
				Attribute a = (Attribute) i.next();

				// get the image URL specified with src 
				if (a.name.equalsIgnoreCase("src"))
				{
					String imgstr = new String(deQuote(a.value));
					if( Tmp.indexOf(imgstr) < 0 )
						Tmp.add(imgstr);
				}
			}
		}

		// get other tags by which a component is inserted in page

		// object inserted with embed/object tags
		if ((t.tagName.equalsIgnoreCase("embed")) || (t.tagName.equalsIgnoreCase("object"))) 
		{
			for (Iterator i=t.attributeList.attributes.iterator(); i.hasNext(); ) 
			{
				Attribute a = (Attribute) i.next();

				// get the image URL specified with src 
				if (a.name.equalsIgnoreCase("src"))
				{
					String imgstr = new String(deQuote(a.value));
					if( Tmp.indexOf(imgstr) < 0 )
						Tmp.add(imgstr);
				}
			}
		}

		// applet on the page
		if (t.tagName.equalsIgnoreCase("applet")) 
		{
			for (Iterator i=t.attributeList.attributes.iterator(); i.hasNext(); ) 
			{
				Attribute a = (Attribute) i.next();

				// get the code from URL specified with code
				if (a.name.equalsIgnoreCase("code"))
				{
					String imgstr = new String(deQuote(a.value));
					if( Tmp.indexOf(imgstr) < 0 )
						Tmp.add(imgstr);
				}
			}
		}

		// StyleSheets used with link tag
		if (t.tagName.equalsIgnoreCase("link")) 
		{
			boolean 	ss = false; // boolean variable to indicate that the link is used for stylesheet

			for (Iterator i=t.attributeList.attributes.iterator(); i.hasNext(); ) 
			{
				Attribute a = (Attribute) i.next();

				if (a.name.equalsIgnoreCase("Stylesheet")) // the link is pointing to style sheet
				{
					ss = true;
				}
				// get the URL of stylesheet  
				if ((a.name.equalsIgnoreCase("href")) && ss)
				{
					String imgstr = new String(deQuote(a.value));
					if( Tmp.indexOf(imgstr) < 0 )
						Tmp.add(imgstr);
				}
			}
		}
	}

	public static String deQuote(String s) 
	{
		if (s.startsWith("\"") && s.endsWith("\""))
			return s.substring(1, s.length() - 1);
		else
			return s;
	}

}
