
/***************************************.
 * The main program for the test Delegate
 * If the Delegate is not registered, Registration process is completed. 
 * The user input is read, if necessary.
 * After the registration is done, other threads such as SendAlive, Scheduler and UploadResult are started.
 *
 */

/**.
 *	This is windows version of MyMain 
 *	In windows, the daemon and the GUI parts are seperate. 
 *	The GUI part is reads the registration details from the user and creates userinfo.conf and proxy.conf files. 
 *	The daemon will run only when these files are already created. 
 *	GUI part is handled by ShowForm class.
 *	In linux, GUI and daemon are together. 
 */

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

public class MyMain
{
	public static void main(String args[])
	{
		// flag to indicate whether registration is done. 
		// delegate operation is not started until registration is complete.
		boolean 	regDone = false;

		// configure various parameters for Delegate
		ConfigDelegate c = new ConfigDelegate();

		while (!regDone)
		{
			// === check whether user info file exist
			// if not, display the form and read the info from user and register the delegate

			final   String BASE_DIR_CONF = new String("C:/PerMos/conf/");
			final String  userInfoFile = new String(BASE_DIR_CONF + "userinfo.conf");

			File usrfl = new File(userInfoFile);
			if (!usrfl.exists()) 
			{
				System.out.println("userinfo.conf does not exist");
				System.out.println("You need to give registration info");
				System.out.println("Please read the documents for details");
				System.exit(0);

		/*		// if userinfo file does not exist, create a form and get info
				Form	f = new	Form();
				f.waitForInfo();

				c.userInfo(f.name,f.email,f.region,f.city,f.link,f.os, f.priv);
				if (f.usePr)
				{
					// proxy is being used
					// call appropriate constructor with parameters
					ConfigProxy cpr	= new ConfigProxy("YES", f.proxyURL, f.proxyPort, f.proxyUsr, f.proxyPass, f.noProxy);
				}
				else // proxy is not used
				{
					ConfigProxy cpr = new ConfigProxy("NO");	
				}
				f.dispose();
				f	= null;
		*/
			} // end if user info file exists

			// calling this function ensures delegate registration
			// if delegate is not registered for some reason, sleep for some time and then again try to register
			regDone = c.regDel();
			if (!regDone)
			{
				try
				{
					Thread.sleep(600000); // sleep for 600 seconds before trying again for registration
				}
				catch (InterruptedException e)
				{

				}
			} // end if not regdone
		}// end while not regDone
		
		// *** Wait for some time before initializing scheduler*** //
		try
		{
			Thread.sleep(60000); // sleep for 600 seconds before initializing scheduler 
		}
		catch (InterruptedException e)
		{

		}
		
		// starting scheduler 
		Scheduler	sch = new Scheduler(); 
	
		// Starting Upload Result thread
		UploadResult u = new UploadResult();
	
		// Start thread to send Alive message
		SendAlive 	sa = new SendAlive();

	}
}
