
/**.
 *	This a wrapper around the socket class. 
 *	For a given URL and port, a socket is constructed. 
 *	In case of an error, it is reported with a predefined format.
 */

import java.net.*;
import java.io.*;

class  OwnSocket 
{
	public Socket 	s;
	public String	sockErr;	// if socket cannot be created, this string gives exact reason for it.

	// Socket time out interval = 20000 milliseconds  = 20 seconds
	// this is the maximum time the socket can block for read.
	int SOCK_TIMEOUT = 20000;	

	// In case time out occurs, the error message to be reported
	String	TOUT_MSG = new String("TIMEOUT");

	/**
	 *  ConnTo is the URL of the page/component
	 *  atPort is the port address for the site	
	 */
	
	OwnSocket(String connTo, int atPort)
	{
		sockErr = new String("");

		try
		{
			if (Global.proxyStatus(connTo) == true)
				s = new Socket(Global.proxyURL, Global.proxyPort);
			else
			{
				String 	serv = new String("");
				int 	port = atPort;
				int 	ind1=0, ind2=0;
				ind1 = connTo.indexOf('/');
				ind2 = 	connTo.indexOf('/', ind1+1);
				ind1 = 	connTo.indexOf('/', ind2+1);
				serv = connTo.substring(ind2 + 1, ind1);
				s = new Socket(serv, port);
			}
		}
		catch(Exception e)
		{

			/**. Get error message correctly. 
			 *	The exception string is
			 *		+ The name of the actual class of this object 
			 *		+ ": " (a colon and a space) 
			 *		+ The result of the getMessage() method for this object 
			 *	We just want the name of the class. Remaining components are removed.
			 *	Also, at the beginning of each class, the hierarchy of the classes is presented. 
			 *	The hierarchy is eliminated and only final class name is retrieved. 
			 *	Eg. If the message is "java.net.UnknownHostException: vsnlprox.iitk.ac.in", 
			 *		only UnknownHostException is retrieved.
			 */

			String err      = new String();
			err             = e.toString();
			String str2     = err.substring(0, err.indexOf(':'));	// remove the colon and further message

			String str3     = new String(str2.substring(str2.lastIndexOf('.')+1)); // remove hierarchy except last component
			sockErr		= str3;	// final error message
		}
		
		try
		{
			s.setSoTimeout(SOCK_TIMEOUT);
		}
		catch(SocketException e)
		{
			sockErr = new String(TOUT_MSG); 
			System.out.println("OwnSocket : Timeout occured ");
			try
			{
				s.close();
			}
			catch (IOException ioe)
			{

			}
		}
	}

	//	This method returns true, if any error has occurred in creating socket
	public boolean isErr()
	{
		if (sockErr.length() > 0)
			return true;
		else 
			return false;
	}

	//	This method returns  the error string
	public String getErr()
	{
		return sockErr;
	}

}	

