/**.
 *	This class contains all the results for a page in transaction. 
 */

import java.io.*;
import java.util.*;

// all the time measurements are in milli seconds

class PageResult
{
	String	PgURL;		// URL of the page 
	int 	completeSize;	// Size of the page including all the components
	int	CompleteDL;	// Download Time for Complete Page(including all the component)
	int	NumComp;	// Total Number of Componets
	ArrayList compRes;	// List of results for each component
	
	boolean	pgSuccess;	// Flag to indicate whether page was fetched successfully - true indicates success
	String	failReason;	// If test has failed, then the string gives the reason for failure


/* 	The contents of the file are stored in a file in case of need for  further processing.
 *	This variable has the name of the file in which the page contents are stored. 
 */

	String 	pgFile;		
	
	PageResult()
	{
		PgURL		= new String("");
		pgFile		= new String("");
		compRes		= new ArrayList();


		// initialize all variable to zero 
		completeSize	= 0;
		CompleteDL	= 0;
		NumComp		= 0;

		pgSuccess	= true;			// start with success
		failReason 	= new String("");	// empty string indicates success
	}


	/**. Add a component result the list
	 */
	
	public void addCmp(ComponentResult cr)
	{
		compRes.add(cr);
	}


	public void setFname(String f)
	{
		pgFile = new String(f);
	}

/*	This function finds the number of components of the page and complete size of page
 *	Size of the page is sum of sizes of all the components
 */
	public void setSize()
	{
		completeSize =	0;
		int I =0;
		ComponentResult CR;
		while (I < compRes.size())
		{
			CR = (ComponentResult) compRes.get(I);
			completeSize +=  CR.cSize ;
			I++;
		}

		NumComp = I ;	
	}

/**. 	If failure occurs, then this method is called with the reason for failure
 *	The reason might be http error code (which is 4XX or 5XX) or
 *	The reason might be string indicating various exceptions such as - ConnectException, NoRouteToHostException, etc.
 *
 */
	public void setFailure(String reason)
	{
		pgSuccess	= false ; 	// failure occurred
		failReason	= reason;	// failure reason string
	}

	public String toString()
	{
		String t = new String("");

		t += "**** \n";
		t += "Page URL	: " + PgURL + "\n";
		t += "Page Delay: " + CompleteDL + "\n";
		t += "Page Size	: " + completeSize+ "\n";
		t += "Page Comp	: " + NumComp + "\n";	
		
		for (int ind=0; ind < compRes.size(); ind++)
		{
			ComponentResult c = (ComponentResult) compRes.get(ind);
			t += c.toString();
		}
		t += "**** \n";
		return t;
	}

	/**.
	 * The HTML code download is stored in a file locally. 
	 * This method delete that file
	 * delPageFile is called when all the processing is completed
	 */

	public boolean delPageFile()
	{
		File 	f = new File(pgFile);
		boolean delPg = false;	// flag to indicate status of deleting page file
		try
		{
			if (f.delete())			// delete the file
			{
				// file got deleted successfully
				delPg = true;
			}
		}
		catch (Exception e)
		{
			System.out.println("PageResult	: Error in Deleting the file " + e );	
			delPg = false;
		}
		return delPg;
	}
}
