
/**  This class manages all the profiles for the jobs, the Delegate is monitoring.
 *   The function getProfile fetches the profies(for given jobs) from Reporting Centre.
 *   Other functions are yet to be decided and implemented. 
 *
 */

// keeps info about all profiles

import java.util.*;
import java.io.*;
import java.text.*;
import java.lang.*;

class Profile
{
	final static String	pfName = new String(Global.getProfileFileName()) ; // profile file name
	
	Profile()
	{

	}

/**
 * Given a list of JobID and JobType, this method will fetch the details of the jobs from Reporting Centre
 */

	synchronized static public void getProfiles(ArrayList P)
	{
		// local variable for the list of jobs for which profile is to be fetched	
		ArrayList 	listp = new ArrayList();
		int 		i=0; //index variable
		listp	= P;

		while (i < listp.size())
		{
			String pgParam 	= new String("");	// page parameters
			String pgParVal	= new String("");	// values for respective page parameters
			String		delim = new String("|");	// delimeter to seperate parameters
			PageDetails	pgDet; 
			PageResult 	pgRes;
			Page		p;
			String 		fName = new String("");

			String 		jid = new String("");
			String 		jtype = new String("");

			PendingJob pj = (PendingJob) listp.get(i);

			jid 	= pj.Jid;		
			jtype 	= pj.JType;
		
			pgParam	  = "JobID" + delim + "JType" + delim;
			pgParVal  = jid + delim + jtype + delim;

			pgDet = Global.getJobDlDetail();
			pgRes = new PageResult();

			pgDet.setPgParam(pgParam, pgParVal,"" );

			p = new Page(pgDet, pgRes);
			try
			{
				p.t.join();
			}
			catch (InterruptedException e)
			{	
				System.out.println("Profile : Thread Interrupted");
			}
			fName = pgRes.pgFile;

			String 	profile = new String(Global.getProfileFileName());

			try
			{
				int 		BUFF_SIZE = 4096;
				char 		buff[] = new char[BUFF_SIZE];
				int 		numBytes = 0;
				char  		newline  = '\n';
			
				// parameter is file name and "true" is for append 
				FileWriter	wrProfile = new FileWriter(profile, true);
				FileReader	rdProfile = new FileReader(fName);
		
				while ((numBytes = rdProfile.read(buff,0,BUFF_SIZE)) > 0)
				{	
					wrProfile.write(buff,0,numBytes);
				}
				wrProfile.write(newline);

				wrProfile.close();
				rdProfile.close();
			}
			catch (Exception e)
			{
				System.out.println("Profile	: Exception in writing page to file " + e);	
			}

			// delete the downloaded file
                        pgRes.delPageFile();
	
			i += 1; 

		}// end while
	}// end getProfiles

/** This method fetches all the jobs and returns the information of the same.
 * 
 */

	synchronized static public ArrayList getListJobs()
	{
		ArrayList 	jlist = new ArrayList();

		try
		{
			BufferedReader input = new BufferedReader(new FileReader(pfName));
			String s = new String("");

			String 	jid = new String("");
			String	jtype = new String("");
			Date 	fD = new Date() ;
			Date 	tD = new Date() ;
			int	stime = 0;
			int 	etime = 0;
			int 	numsamp = 0;
			int 	numsteps = 0;

			while((s=input.readLine())!= null) 
			{
		
				if (s.startsWith("#JPROFILE"))
				{

				}

				if (s.startsWith("JOBID"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					jid = st.nextToken();
				}

				if (s.startsWith("TYPE"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					jtype = st.nextToken();
				}
				if (s.startsWith("FROM"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					String dateStr = new String(st.nextToken()); // get date string

					// convert date string to Date object
					String inputPattern = "yyyy-MM-dd";	

					//  *** Set up formatter for input Date string (wrt GMT)
					SimpleDateFormat 	inputDateformatter = new SimpleDateFormat();
					TimeZone		tz =  TimeZone.getTimeZone("GMT");
					inputDateformatter.setTimeZone(tz);				
					inputDateformatter.applyPattern(inputPattern);

					//  *** Parse into a Date object
					ParsePosition pos = new ParsePosition(0);
					fD = inputDateformatter.parse(dateStr, pos);
				}
				if (s.startsWith("TO"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					String dateStr = new String(st.nextToken()); // get date string

					// convert date string to Date object
					String inputPattern = "yyyy-MM-dd";	

					//  *** Set up formatter for input Date string (wrt GMT)
					SimpleDateFormat inputDateformatter = new SimpleDateFormat();
					TimeZone		tz =  TimeZone.getTimeZone("GMT");
					inputDateformatter.setTimeZone(tz);				
					inputDateformatter.applyPattern(inputPattern);

					//  *** Parse into a Date object
					ParsePosition pos = new ParsePosition(0);
					tD = inputDateformatter.parse(dateStr, pos);
				}

				if (s.startsWith("START"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					stime = Integer.parseInt(st.nextToken());
				}

				if (s.startsWith("END"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					etime = Integer.parseInt(st.nextToken());
				}

				if (s.startsWith("NUMSAMP"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					numsamp = Integer.parseInt(st.nextToken());
				}

				if (s.startsWith("NUMSTEP"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					numsteps = Integer.parseInt(st.nextToken());
				}

				if (s.startsWith("#JSTOP")) // add the jobinfo object to the list
				{
					JobInfo j = new JobInfo(jid, jtype, fD, tD, stime, etime, numsamp ,numsteps);

					// ***  check if the job is already in the list
					boolean isPresent = false;
					int     ind       = 0;

					while(ind < jlist.size())
					{
						JobInfo jtmp = (JobInfo) jlist.get(ind);

						// try matching jobid and jobtype
						if (jtmp.JobID.equals(j.JobID) && jtmp.Jtype.equals(j.Jtype))
						{
							isPresent = true;       // match found
							System.out.println("Profile     : Already present "+j.JobID);
						}
						ind += 1;
					}

					// if the job does not exist in the list, add it.
					if (isPresent == false)
						jlist.add(j);
				}

			}// end while input readline
			input.close();

		}// end try
		catch(IOException e) 
		{
			System.out.println("Profile : IOException" + e );
	    	}

		return jlist;
	}

/**.
 *	This method gives the details of the job for given Job ID. 
 *	It accepts the JobID and Job Type as parameter. 
 *
 *	The profile file is searched for given JobID. All the details are retrieved. 
 *	The method to retrieve Datasets:
 *
 */
	synchronized static public JobDetails  getJobDetails(String jID, String jType)
	{
		JobDetails jD	= new JobDetails();

		try
		{
			BufferedReader input = new BufferedReader(new FileReader(pfName));
			String s = new String("");

			String 	jid = new String("");		// Job ID
			String	jtype = new String("");		// job type

			String 	hMethod	= new String("");	// HTTP Method
			String 	hVersion = new String("");	// HTTP Version
			String	pgURL = new String("");		// Page URL
			int 	pgPort = 0;			// Page Port
			String	pgParam = new String("");	// Page Parameters
			String	pgData	= new String("");	// Page Data	
			String	pgKeyword = new String("");	// Page Keywords

			/*	data set number to be used for this test
			 * 	For a job, there can be multiple data sets. 
			 *	From given sets, one is chosen randomly. The data set number which is chosen, is indicated by this variable
			 *	Initialized to zero to indicate no data sets
			 */
			int 	dSetNum = 0;	

			boolean	done = false;		// flag to indicate, reading from file done.
			boolean	inside = false;		// flag to indicate, currently reading the job info for given ID


			// read from file
			while (((s=input.readLine())!= null) && (!done))
			{
				if (s.startsWith("#JPROFILE"))
				{
					

				}

				if (s.startsWith("#JEN"))	// end of job detail
				{
					if (inside)
					{
						done = true;
						// when the job details are filled completely, write the JobID
						// in case of some problem, this field is left blank, indicating failure
						jD.JobID	= jID; 
					}
					else
						inside = false;
					continue;
				}

				if (s.startsWith("JOBID"))
				{
					StringTokenizer st = new StringTokenizer(s);
					st.nextToken();	// skip first token
					jid = st.nextToken(); // got JobID

					// get job Type
					s = input.readLine();
					if (s.startsWith("TYPE"))
					{
						// StringTokenizer st = new StringTokenizer(s);
						st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						jtype = st.nextToken();
					}

					// match job ID and job type
					if (jid.equalsIgnoreCase(jID) && jtype.equalsIgnoreCase(jType))
					{
						inside = true;
						continue;
					}
				}

				if (inside)
				{
					if (s.startsWith("NUMSET"))
					{
						int numSet;
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						numSet = Integer.parseInt(st.nextToken());	// get number of data sets

						if (numSet > 0)
						{
							// data set will be selected only if there is some data set (numset > 0 )
							// generate a random number between 1 and "number of data sets"
							dSetNum =  1 + ((int) (Math.random() * 100)) % numSet; 
						}
					}

					if (s.startsWith("HTTPM"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						hMethod = st.nextToken();
					}

					if (s.startsWith("HTTPV"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						hVersion = st.nextToken();
						
					}

					if (s.startsWith("URL"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						pgURL = st.nextToken();
					}
					if (s.startsWith("PORT"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token
						pgPort = Integer.parseInt(st.nextToken());
					}

					if (s.startsWith("PARAM"))
					{
						StringTokenizer st = new StringTokenizer(s);
						st.nextToken();	// skip first token

						String tok1 = new String("");	// this token shoule be '{'
						String tok2 = new String("");	// this is dataset string
						String tok3 = new String("");	// this token should be '}'

						tok1 	= st.nextToken(); 
						tok2 	= st.nextToken(); 
						tok3 	= st.nextToken(); 

						if (tok1.equalsIgnoreCase("{") && tok3.equalsIgnoreCase("}"))
						{
							// if  parameter field is not empty, read it
							if (tok2.equalsIgnoreCase("NONE") == false) 
								pgParam = tok2;
						}
					}

					if (s.startsWith("DATASET"))
					{
						if (dSetNum > 0)
						{
							int num;
							StringTokenizer st = new StringTokenizer(s);
							st.nextToken();	// skip first token
							num = Integer.parseInt(st.nextToken()); // get the data set number
							if (dSetNum == num) // correct data set found
							{	// read next 3 tokens
								String tok1 = new String("");	// this token shoule be '{'
								String tok2 = new String("");	// this is dataset string
								String tok3 = new String("");	// this token should be '}'
		
								tok1 	= st.nextToken(); 
							//	tok2 	= st.nextToken(); 
							//	tok3 	= st.nextToken(); 

								if (tok1.equals("{"))
								{
									tok2    = st.nextToken();
									if (tok2.equalsIgnoreCase("NONE") == false)
										pgData = tok2;

									while (st.hasMoreTokens())
									{
										String tmptok = new String("");
										tmptok = st.nextToken();
										if (tmptok.equals("}") == false)
											pgData += " " + tmptok ;
									}//end while
								}//end if tok1 == {

			
/*								if (tok1.equalsIgnoreCase("{") && tok3.equalsIgnoreCase("}"))
								{
									// if data set is not empty, read it
									if (tok2.equalsIgnoreCase("NONE") == false) 
										pgData = tok2;
								}
*/
							}
						}
					}

					if (s.startsWith("KEYWORD"))
					{
						if (dSetNum > 0)
						{
							int num;
							StringTokenizer st = new StringTokenizer(s);
							st.nextToken();	// skip first token
							num = Integer.parseInt(st.nextToken()); // get the keyword set number
							if (dSetNum == num) // correct keyword set found
							{	// read next 3 tokens
								String tok1 = new String("");	// this token shoule be '{'
								String tok2 = new String("");	// this is keyword string
								String tok3 = new String("");	// this token should be '}'
		
								tok1 	= st.nextToken(); 
								tok2 	= st.nextToken(); 
								tok3 	= st.nextToken(); 

								if (tok1.equalsIgnoreCase("{") && tok3.equalsIgnoreCase("}"))
								{
									// if keyword set is not empty, read it
									if (tok2.equalsIgnoreCase("NONE") == false) 
										pgKeyword = tok2;
								}
							}

						}
					}

					if (s.startsWith("#ENDPAG")) // end of page
					{
						// add the read page details to transaction details
						PageDetails pD	= new PageDetails(hMethod, hVersion, pgURL, pgPort, pgParam, pgData, pgKeyword); 

						jD.ListPageDetails.add(pD);

						//clear all the fields
					}
				}
			}// end while 
			input.close();
		}// end try
		catch(IOException e) 
		{
			System.out.println("Profile : IOException" + e);
	    	}

		return jD;
	} // end getJobDetails

/*
 *	purgeExpired() deletes the profiles for which testing period has expired. 
 *	This method is called by Scheduler.
 *	The parameters are the Job ID and Job type
 */
 
	synchronized static public void purgeExpired(String jid, String jtype)
	{
		System.out.println("Purging " + jid + "  " + jtype);
		
		String 	tmpFile = new String("monitor.tmp");

		// create temporary file for storing result 
		File f = new File(tmpFile);
		if (f.exists() == false)	// if file does not exit, create a new one.
		{
			try
			{
				f.createNewFile();
			}
			catch (Exception e)
			{
				System.out.println(" Error in creating file : Profile.java " + e );
			}
		}

		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(tmpFile));
			BufferedReader in = new BufferedReader(new FileReader(pfName));

			String 	s	= new String("");
			boolean inside	= false;	// flag to indicate whether current line is inside uploaded result

			String	tmpJid	= new String("");
			String	tmpJtype= new String("");
			
			// ***	copy contents of result file in temp file except the uploaded result
			while((s=in.readLine())!= null) 
			{
				// new profile started
				if (s.startsWith("#JPRO"))
				{
					// next line gives the JobID
					String s2 = in.readLine();

					// next line gives JobType
					String s3 = in.readLine();

					StringTokenizer st = new StringTokenizer(s2);
					st.nextToken();			// this token contains "JOBID"
					tmpJid = st.nextToken();	// get JobID from the string
					
					st = new StringTokenizer(s3);
					st.nextToken();			// this token contains "TYPE"
					tmpJtype = st.nextToken();	// get job type from the string

					if (tmpJid.equalsIgnoreCase(jid) && tmpJtype.equalsIgnoreCase(jtype))
					{
						// match found 
						inside = true;
						continue;
					}
					else
					{
						out.write(s,0,s.length());
						out.newLine();
						out.write(s2,0,s2.length());
						out.newLine();
						out.write(s3,0,s3.length());
						out.newLine();
						continue;
					}
				}

				if (s.startsWith("#JEN"))
				{
					if (inside)
					{
						inside = false;
						continue;
					}
				}
				if(!inside)
				{
					out.write(s,0,s.length());
					out.newLine();
				}
			}
			out.flush();
			out.close();
			in.close();

			// *** delete the profile file and rename temporary file as profile file
			
			f		= new File(pfName);
			File	tF	= new File(tmpFile);

			if (f.delete())		// delete the file
			{
				// file got deleted successfully
				tF.renameTo(f);	// rename the temporary file
			}

		}
		catch (Exception e)
		{	
			System.out.println("Profile	: Error in file IO " + e );	
		}
		
	} // end purgeExpired
}
