
/** 
 *	Scheduler has list of all the jobs assigned to delegate. 
 *	Its main function is to schedule various jobs at proper time.
 * 
 *	NOTE: Most of the messages are commented out. In case you want to debug the program just remove
 *	comments from the System.out.println.
 *	
 */

import java.util.*;
import java.lang.*;

class Scheduler extends Thread
{
	Thread 		t;
	ArrayList	listJ;
	long 		MAX_WAIT = 180000;	// Maximum time (in milliseconds) to wait before stoping a thread.

	Scheduler()
	{
		t = new Thread(this);
		t.setName("Scheduler");
		listJ = new ArrayList();
	
		// ***** initialize the job list
		listJ = Profile.getListJobs();
		
		int	ind = 0;
		while (ind <listJ.size())
		{
			JobInfo j = (JobInfo) listJ.get(ind);

			// calculate the interval for testing 
			// interval is in minutes 
			// interval = duration of test in minutes / number of samples

			j.interval = (j.EndTime - j.StartTime)*60/(j.NumSample); 

			// Set scheduling parameters

			// get the time for scheduling next 
			j.NextSch = getNextSchedule(j.FromDate, j.ToDate, j.StartTime, j.EndTime);

			// set the same time for actual scheduling of next sample
			j.ActualNext = j.NextSch;


			// *** first make sure that none the transaction has expired the period
			// if some job has expired its scheduling period, delete it from the list

			long trEnd	= j.ToDate.getTime() + (j.EndTime * 60* 60* 1000);

			if (j.NextSch.getTime() > trEnd)
			{
				Profile.purgeExpired(j.JobID, j.Jtype);
				listJ.remove(ind);
				continue;
			}
			ind++;
		}// end while
	
		System.out.println("Scheduler: No. of schedulable jobs : "+listJ.size());

		t.start();
	}

	public void run()
	{
		int ind=0;
			
		while (true)
		{
			// if there are no jobs to be scheduled, sleep
			System.out.println("Scheduler: No. of schedulable jobs : "+listJ.size());

			while (listJ.size() == 0)
			{
				try
				{
					System.out.println("Scheduler:	No jobs to schedule. Sleeping for 600 seconds ");

					// Reclaim the memory with garbage collector
					System.gc();		
					Thread.sleep(600000);		// Wait for some time 
					getNewJob(listJ);		// Fetch new jobs
					System.out.println("Scheduler: Number of jobs "+ listJ.size());
				}
				catch (InterruptedException e)
				{
					System.out.println("Scheduler Interrupted  while sleeping for job " + e + "\n");
				}
			} // end while

			JobInfo j	= new JobInfo();
			Date	cur	= new Date();

			j = (JobInfo) listJ.get(ind);

			// if current time has exceeded the actual scheduling time, schedule the job
			if (j.ActualNext.compareTo(cur) <= 0 )
			{

				// *** get all the parameters for this job 
				// *** schedule this job 

				JobDetails jD = new JobDetails();
				jD		= Profile.getJobDetails(j.JobID, j.Jtype);

				System.out.println("Scheduler:	Scheduling the job	\n"+ jD.toString());

		/*	Idea of Multiple Testing:
		 *	Normally it is observed that some failures in accessing webpages are purely of transient in nature.
		 *	It might be due to instantaneous problems in network.
		 *	Such transient failures should be avoided completely, so as to get close picture of website's response time. 
		 *	Such transient errors can be removed if multiple tests are performed within a short interval of time
		 *	The two tests are executed after a given interval (currently 30 seconds). 
		 *	The results of these two tests are then merged to get the final result of the test.
		 *
		 */
				// start first test
				Transaction t1 = new Transaction(jD);
				// Wait for transactions to finish 
				try
				{
					t1.t.join();
				}
				catch (InterruptedException e)
				{
					System.out.println("Scheduler Interrupted wating for transaction to finish " + e + "\n");
				}
				// get results
				TransResult tr1 = t1.getResult(); 
				System.out.println("Scheduler: Completed  "+ tr1.JobID);

				// start second test
				Transaction t2 = new Transaction(jD);
				
				try
				{
					t2.t.join();
				}
				catch (InterruptedException e)
				{
					System.out.println("Scheduler Interrupted wating for transaction to finish " + e + "\n");
				}

				TransResult tr2 = t2.getResult();
				System.out.println("Scheduler: Completed  "+ tr2.JobID);

				// merge the two results 
				TransResult trFinal = tr1.getAvg(tr2);

				// Submit the final result
				Result.submitTr(trFinal);

				// Delete all the page files
				tr1.delTrFile();
				tr2.delTrFile();

				/*
				System.out.println("Scheduler:	Scheduling the job	"+ j.JobID);
				System.out.println("Scheduler:	NextSch			"+ j.NextSch);
				System.out.println("Scheduler:	Actual Next		"+ j.ActualNext);
				*/

				/*	System.out.println("Scheduler: "+ j.JobID);
				 *	System.out.println("Scheduler: "+ j.Jtype);
				 *	System.out.println("Scheduler: "+ j.FromDate);
				 *	System.out.println("Scheduler: "+ j.ToDate);
				 *	System.out.println("Scheduler: "+ j.StartTime);
				 * 	System.out.println("Scheduler: "+ j.EndTime);
				 * 	System.out.println("Scheduler: "+ j.NumSample);
				 *	System.out.println("Scheduler: "+ j.NumSteps);
				 *	System.out.println("Scheduler: ****** ");
				 */

				// Update scheduling parameters

				// find out time for scheduling next test
				j.NextSch	= new Date(j.NextSch.getTime() + (j.interval * 60 * 1000)); 

				/**. 	The transaction does not start at time indicated by NextSch, 
				 *	but few minutes before or after the scheduled time
				 *	This is needed to make sure the execution of tests are spread across the given time period.
				 *	It gives the better picture of the website behaiour over the period of time.
				 */
				
				// the range is 30% of interval
				int low		= - (int) (j.interval * 0.3);		
				int high	= (int) (j.interval * 0.3); 

				// calculate the random amount of time (in minutes0 that should be deducted from or added to scheduled time
				int delta	= low + 1 + (int) (Math.random() * (high-low-1));
				
				j.ActualNext	= new Date(j.NextSch.getTime() + (delta * 60 * 1000));

				/*
				System.out.println("Scheduler:	Updating parameters ..	");
				System.out.println("Scheduler:	NextSch			"+ j.NextSch);
				System.out.println("Scheduler:	Actual Next		"+ j.ActualNext);
				*/

			/*	
				System.out.println("Scheduler: ****** ");
				System.out.println("Scheduler: jobid 	"+ j.JobID);
				System.out.println("Scheduler: jobtype	"+ j.Jtype);
				System.out.println("Scheduler: from date "+ j.FromDate);
				System.out.println("Scheduler: to date	"+ j.ToDate);
				System.out.println("Scheduler: start time "+ j.StartTime);
				System.out.println("Scheduler: end time	"+ j.EndTime);
				System.out.println("Scheduler: samples	"+ j.NumSample);
				System.out.println("Scheduler: steps	"+ j.NumSteps);
				System.out.println("Scheduler: interval	"+ j.interval);
				System.out.println("Scheduler: nextSch	"+ j.NextSch);
				System.out.println("Scheduler: actualSch"+ j.ActualNext);
				System.out.println("Scheduler: ListSize "+ listJ.size());
				System.out.println("Scheduler: ****** ");
			*/
			} // end if 
	
			// check if this job has exceeded its scheduling period
			// if yes, delete the current job from the list

			long trEnd	= j.ToDate.getTime() + (j.EndTime * 60* 60* 1000);

			if (j.NextSch.getTime() > trEnd )
			{
				listJ.remove(ind);
				Profile.purgeExpired(j.JobID, j.Jtype);
				System.out.println("Scheduler : Job Period Expired. Removing job "+j.JobID);
				continue;
			}

			ind += 1;
			ind %= listJ.size();

			// sleep till next event
			int ind2 = 0; // index variable
			long diff = 0; // difference between current time and next nearest scheduling time (in milliseconds)

			// intialize diff to be equal to time lag for first job

			Date	today = new Date();
			long	tm = today.getTime();

			JobInfo jtmp	= new JobInfo();
			jtmp		= (JobInfo) listJ.get(ind2);
			diff 		= jtmp.ActualNext.getTime() - tm;

/*
System.out.println("Scheduler : ****** ");
System.out.println("Scheduler : jobid	     " + jtmp.JobID );
System.out.println("Scheduler : today        " + today );
System.out.println("Scheduler : actual next  " + jtmp.ActualNext );
System.out.println("Scheduler : difference   " + diff );
System.out.println("Scheduler : ****** ");
*/
		
			ind2 += 1;

			while (ind2 < listJ.size())
			{
				JobInfo j2	= new JobInfo();

				j2 = (JobInfo) listJ.get(ind2);

				long	tmp = j2.ActualNext.getTime() - tm;
/*
System.out.println("Scheduler : ****** ");
System.out.println("Scheduler : ind2	     " + ind2 );
System.out.println("Scheduler : jobid	     " + j2.JobID );
System.out.println("Scheduler : today        " + today );
System.out.println("Scheduler : actual next  " + j2.ActualNext );
System.out.println("Scheduler : difference   " + tmp );
System.out.println("Scheduler : ****** ");
*/
				if (tmp < diff)
					diff = tmp;
			
				ind2++;
			} // end while ind2

			// if difference is some positive quantity, sleep for those many millisecond
			if (diff > 0)
			{
				System.out.println("Scheduler : sleeping for " + diff + " milliseconds");
				try
				{
					Thread.sleep(diff);
					getNewJob(listJ);
					System.out.println("Scheduler: Number of jobs "+ listJ.size());
				}
				catch (InterruptedException e)
				{
					System.out.println("Scheduler Interrupted " + e + "\n");
				}
			} // end if

		} // end while ind < listJ.size
	}

	/** schNow() accepts the time interval ( date and time)  for execuring the job.
	 *	if Current time falls in beetween this interval, the transaction is to be scheduled. hence true is returned.  
	 *	if Current time falls outside this range, false is returned.
	 *
	 *	Date d1 gives start date and d2 gives end date
	 *	t1 gives the start hour of the day whereas t2 gives end hour of day
	 *	
	 */

	boolean schNow(Date d1, Date d2, int t1, int t2)
	{
		Date curD	= new Date();		// current date
		long curT	= curD.getTime();	// get current timestamp

		// calculate time stamp for  transaction start time
		// add the start hour(in milliseconds) to the date to get correct transaction start time
		// eg. if the start hour is 01 then add milliseconds equivalent to 01 hour.

		long trStart	= d1.getTime() + (t1* 60* 60* 1000);
		long trEnd	= d2.getTime() + (t2* 60* 60* 1000);
		
		if ((curT < trStart) || (curT > trEnd))
		{
			// current time is completely out of range 
			return false;
		}
	
		// calculate the time duration for one day
		long dayStart	= trStart;
		long dayEnd	= trStart + (t2 * 60* 60* 1000);
	
		while (dayEnd <= trEnd )
		{
			// if current time is between start and end hours of the day
			if ((dayStart <= curT) && (curT <= dayEnd ))
				return true;

			// get time stamps for next day	
			dayStart	+= 24*60*60*1000;
			dayEnd		+= 24*60*60*1000;
		}

		// current time is not in schedule period, hence return false
		return false; 
	}

	/** getNextSch() method returns time stamp for next timestamp for scheduling the job.  
	 *	This method is invoked
	 *	1. at the intialization of the job list
	 *	2. when current time is out of range of schedulable period
	 */

	Date getNextSchedule (Date d1, Date d2, int t1, int t2)
	{
		Date curD	= new Date();		// current date
		long curT	= curD.getTime();	// get current timestamp

		// calculate time stamp for  transaction start time
		// add the start hour(in milliseconds) to the date to get correct transaction start time
		// eg. if the start hour is 01 then add milliseconds equivalent to 01 hour.

		long trStart	= d1.getTime() + (t1* 60* 60* 1000);
		long trEnd	= d2.getTime() + (t2* 60* 60* 1000);

		// transaction start time is ahead of current time
		if (curT < trStart)
		{
			Date nxtD	= new Date(trStart); // schedule the transaction at start time of transaction
			return nxtD;
		}

		// transaction testing period is over. just send current time
		if (curT > trEnd)
		{
			return curD;
		}

		// *** current time falls in transaction testing period

		// calculate the time duration for one day
		long dayStart	= trStart;
		long dayEnd	= trStart + (t2 * 60* 60* 1000);
	
		while (dayEnd <= trEnd )
		{
			// if current time is between start and end hours of the day, return current time
			// which means the transaction should be started immediately
			if ((dayStart <= curT) && (curT <= dayEnd))
				return curD;

			// get time stamps for next day	
			dayStart	+= 24*60*60*1000;

			// if current time is in between end of today's hour and start of next day's hour, 
			// 	use start day of next day.
			if ((curT <= dayStart) && (dayEnd <= curT))
			{
				Date nxtD	= new Date(dayStart); // schedule the transaction at start time of next day
				return nxtD;
			}

			// time stamp for end of next day
			dayEnd		+= 24*60*60*1000;
		}
		return curD;
	}

	/*	getNewJob() gives a list of jobs to be monitored.
	 *	It accpets the current Job List as parameter. A list of all jobs is fetched from Profile. 
	 *	This method then combines both the lists and gives a unified list
	 *	Initially, there will not be any jobs, hence the list cur will be empty.
	 *	
	 */
	
	private void getNewJob(ArrayList cur)
	{
		ArrayList tmpNew = Profile.getListJobs();
		
		ArrayList resList = new ArrayList();	// the resulting array of JobInfo

		int ind = 0;		// index for tmpNew list
		boolean addCur = false;	// flag to indicate whether current jobinfo should be added to the list of jobs

		while(ind < tmpNew.size())
		{
			JobInfo jTmp = (JobInfo) tmpNew.get(ind);

			// find the match for this job, if any.

			int 	ind2 = 0;		// index for cur
			boolean match = false;		// flag to indicate JobID match is found
			while ((ind2 < cur.size()) && (!match))
			{
				JobInfo	jCur = (JobInfo) cur.get(ind2);
				if (jCur.JobID.equalsIgnoreCase(jTmp.JobID)) // match found 
					match = true;

				ind2++;
			}// end while ind2

			// no match found, add to the list of jobs
			if (!match)
			{
				addCur = true;
				// calculate the interval for testing 
				// interval is in minutes 
				// interval = duration of test in minutes / number of samples

				jTmp.interval = (jTmp.EndTime - jTmp.StartTime)*60/(jTmp.NumSample); 

				// Set scheduling parameters

				// get the time for scheduling next 
				jTmp.NextSch = getNextSchedule(jTmp.FromDate, jTmp.ToDate, jTmp.StartTime, jTmp.EndTime);

				// set the same time for actual scheduling of next sample
				jTmp.ActualNext = jTmp.NextSch;


				// *** first make sure that none the transaction has expired the period
				// if some jTmpob has expired its scheduling period, delete it from the list

				long trEnd	= jTmp.ToDate.getTime() + (jTmp.EndTime * 60* 60* 1000);

				if (jTmp.NextSch.getTime() > trEnd)
				{
					Profile.purgeExpired(jTmp.JobID, jTmp.Jtype);
					addCur = false;		// this job should not be added as it has expired its period
				}
				
				System.out.println(jTmp.toString());
				
				cur.add(jTmp);
				
				System.out.println("Scheduler	: Number of jobs " + cur.size());
				
				resList.add(jTmp);

			} // end if not match
			ind++;
		} // end while ind
	}
}
