
/**. 
 *  This class represents a  Transaction for a website.
 *  The input is an object JobDetails object which has information about each step in transaction. 
 *  For each step of transaction, the details for that step (Page) are fetched from JobDetails. 
 *  The details of the page alongwith a result object is given to Page. 
 *  Each page returns updated result object.
 *  The pages are invoked sequentially.
*/ 

import java.util.*;
import java.text.*;
import java.lang.*;

class Transaction extends Thread
{
	Thread 		t; 
	JobDetails 	TrDetail;	// Details of the transaction - inludes list of pages to be fetched
	PageDetails 	PgDetail; 	// Details of each page to be fetched
	TransResult 	TrRes;		// Restults of the complete transaction
	PageResult 	PgRes;		// Result for one page or step of transaction

	boolean 	done = false;	// Flag to indicate completion of transaction
	boolean		isWaiting=false;// Flag to indicate wait condition on the event "transaction completed"	

	// Time out value for a page. If complete page is not received in this time, it is taken as time out. 
	// The time duration is in milliseconds. 
	// Currently it is set to 2 minutes
	long		TIME_OUT = 120000;

	// In case time out occurs, the error message to be reported
	String		TOUT = new String("TIMEOUT");
	
	Transaction(JobDetails T)
	{
		t = new Thread(this);
		t.setName("Transaction");
		TrDetail = T;
		t.start();
	}

	public void run()
	{
		Date 			D1;		// variable to measure the end-to-end response time of transaction
		long 			T1, T2;		// variables for time measurement
		Date			dgmt;		// date for noting start of transaction
		String			stTime;		// timestamp of start of transaction
		SimpleDateFormat	sfd;		// Representing the Date in proper format
		TimeZone 		tz; 		// Time zone to be set to GMT
		String 			jid;		// JobID

	/**	All the dates and time stamps are converted to GMT for the communication between Delegate and Reporting Centere.
	 *	Communication in GMT removes the ambiguity regarding all the time related parameters. 
	 *	Hence we need to normalise the local date/time to GMT.
	 */

		// Format of Date: "Year-Month-Date Hour:Minute:Seconds"
		sfd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		tz = TimeZone.getTimeZone("GMT");	// time zone is GMT
		sfd.setTimeZone(tz);
		dgmt = new Date();
		stTime = new String(sfd.format(dgmt));	// note down start of transaction 

		jid = new String(TrDetail.getJID());

		TrRes = new TransResult(jid, stTime);	// Transaction result object

		D1 = new Date();
		T1 = D1.getTime();

		int i=0;	// index variable
		while (i < TrDetail.ListPageDetails.size())
		{
			PgRes = new PageResult();	// Transaction result object
			PgDetail = (PageDetails)TrDetail.ListPageDetails.get(i); // Retrieve page details for each step	

			i += 1;	// increment i so that it will indicate the step number of page being fetched

			// create new Page object PageDetails and PageResults objects
			// PageResult object will be filled with resulting parameters for the page and its components
			// start a new thread to fetch the page
			Page PgFetch = new Page(PgDetail, PgRes); 
			try
			{
				PgFetch.t.join(TIME_OUT); // wait for  120 seconds (2 minutes)
			}
			catch (InterruptedException e)
			{
				System.out.println(" Thread Interrupted");
			}
	
			if (PgFetch.t.isAlive())  // Time out has occurred because thread is still alive
			{
				System.out.println("Page : Timeout occurred ");
				PgRes.setFailure(TOUT); // Report time out failure
				PgFetch.t.interrupt();
				PgFetch = null;
			}
			TrRes.ListPageRes.add(PgRes);

			// if there is any error in fetching this page, abort the transaction reporting failure
			if (!PgRes.pgSuccess) 
			{
				TrRes.setFailure(i); 	// failure occured at step i
				break;			// don't continue the transaction
			}
		} // end while
		D1 = new Date();
		T2 = D1.getTime();

		// numbe of steps in transaction is equal to the number of pages to be in the list of page details
		TrRes.NumSteps = TrDetail.ListPageDetails.size();
		TrRes.EtoE = (int) (T2-T1);
	
		return;
	}//end run

	synchronized public TransResult getResult()
	{
		return TrRes;
	}

synchronized private void trdone()
	{
		done = true;
		if (isWaiting)
		{
			notify();	
		}

	}
}//end Transaction

	// Extra code which might be used to print some results
	// print the results for the transaction, one page at a time. 
	/*	i=0;
		while (i< TrRes.ListPageRes.size()) 	
		{
			PgRes = (PageResult) TrRes.ListPageRes.get(i);
		
			
			System.out.println("******  ");
			System.out.println("Step " + (i+1));
			System.out.println("URL " + PgRes.PgURL );
			System.out.println("size "+ PgRes.CSize  );
			System.out.println("time "+ PgRes.CompleteDL + " ms" );
			System.out.println("******  ");


			// print the results for all the components
			int I =0;
			ComponentResult CR;
			System.out.println("Component of the page ");
			while (I < PgRes.compRes.size())
			{
				//System.out.println(" printing results for components ");
				//System.out.println("~~~~");
				CR = (ComponentResult) PgRes.compRes.get(I);
				System.out.println("\t Component "+ (I+1) + " URL   " + CR.cmpURL);
				System.out.println("\t Download time  " + CR.cmpDL+ " ms");
				System.out.println("\t Component Size " + CR.cSize );
				System.out.println("\t Header Size    " + CR.headerSize);
				System.out.println("");
				I++;
			}	
			i++;	// get next component
		}
	*/

